"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _symbol = require("babel-runtime/core-js/symbol");

var _symbol2 = _interopRequireDefault(_symbol);

var _getIterator2 = require("babel-runtime/core-js/get-iterator");

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _map = require("babel-runtime/core-js/map");

var _map2 = _interopRequireDefault(_map);

var _from = require("babel-runtime/core-js/array/from");

var _from2 = _interopRequireDefault(_from);

exports.storeArguments = storeArguments;
exports.evaluateExpression = evaluateExpression;
exports.evaluateExpr = evaluateExpr;
exports.evaluatePreview = evaluatePreview;
exports.evaluateEntity = evaluateEntity;
exports.ensureValue = ensureValue;
exports.getValue = getValue;
exports.evaluateEntityTree = evaluateEntityTree;
exports.globalEntity = globalEntity;
exports.evaluate = evaluate;

var _List = require("fable-core/umd/List");

var _List2 = _interopRequireDefault(_List);

var _babel = require("../common/babel");

var _babel2 = require("../../bindings/babel");

var _common = require("../../libraries/common");

var _series = require("../../libraries/series");

var _AsyncBuilder = require("fable-core/umd/AsyncBuilder");

var _runtime = require("../codegen/runtime");

var _charts = require("../../libraries/google/charts");

var _tables = require("../../libraries/tables");

var _general = require("../../libraries/general");

var _interactive = require("../../libraries/interactive");

var _ast = require("../ast/ast");

var _astops = require("../ast/astops");

var _Util = require("fable-core/umd/Util");

var _Seq = require("fable-core/umd/Seq");

var _Lazy = require("fable-core/umd/Lazy");

var _Lazy2 = _interopRequireDefault(_Lazy);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function storeArguments(values) {
    return [(0, _from2.default)(values), (0, _List.mapIndexed)(function (i, _arg1) {
        return new _babel.Expression("MemberExpression", [new _babel.Expression("IdentifierExpression", ["_stored", null]), new _babel.Expression("NumericLiteral", [i, null]), true, null]);
    }, values)];
}

function evaluateExpression(_stored, expr) {
    var prog = new _babel.Program(null, (0, _List.ofArray)([new _babel.Statement("ExpressionStatement", [expr, null])]));
    var code = Babel.transformFromAst(_babel.Serializer.serializeProgram(prog), "", new _babel2.BabelOptions(["es2015"]));

    _common.Log.trace("interpreter", "Interpreter evaluating: %O using values %O", code.code, _stored);

    try {
        var s = _series.series.create(function (builder_) {
            return builder_.Delay(function () {
                return builder_.Return([]);
            });
        }(_AsyncBuilder.singleton), "", "", "");

        new _runtime.RuntimeContext("lol", "", "troll");

        (function (c) {
            return function (s_1) {
                return (0, _runtime.trimLeft)(c, s_1);
            };
        });

        (function (f) {
            return function (g) {
                return function (data) {
                    return (0, _runtime.convertTupleSequence)(f, g, data);
                };
            };
        });

        (function (f_1) {
            return function (data_1) {
                return (0, _runtime.convertSequence)(f_1, data_1);
            };
        });

        (function (arg00) {
            return _charts.chart.bar(arg00);
        });

        _tables.table.create(s);

        _general.date.now();

        _series.series.values(new Int32Array([1]));

        _tables.placeholder.create("");

        (function (arg00_1) {
            return _interactive.youguess.line(arg00_1);
        });

        _stored.length;
        return eval(code.code);
    } catch (e) {
        _common.Log.exn("interpreter", "Evaluation failed: %O", e);

        throw new Error();
    }
}

function evaluateExpr(args, exprBuilder) {
    var patternInput = storeArguments(args);
    return evaluateExpression(patternInput[0], exprBuilder(patternInput[1]));
}

function evaluatePreview(ent, value) {
    var previewName = new _ast.Name("preview");

    _common.Log.trace("interpreter", "Evaluating preview on: %O (%s)", ent, (0, _astops.formatType)(ent.Type));

    var matchValue = ent.Type;
    var $var389 = void 0;

    if (matchValue != null) {
        if (matchValue.Case === "Object") {
            var activePatternResult7108 = function (obj) {
                return (0, _astops.$7C$FindMember$7C$_$7C$)(previewName, obj);
            }(matchValue.Fields[0]);

            if (activePatternResult7108 != null) {
                $var389 = [0, activePatternResult7108];
            } else {
                $var389 = [1];
            }
        } else {
            $var389 = [1];
        }
    } else {
        $var389 = [1];
    }

    switch ($var389[0]) {
        case 0:
            if ($var389[1].Type.Case === "Method") {
                return evaluateExpr((0, _List.ofArray)([value]), function (inst) {
                    return _babel.BabelOperators.op_DivideAtDivide($var389[1].Emitter.Emit(inst.head), new _List2.default());
                });
            } else {
                return evaluateExpr((0, _List.ofArray)([value]), function (inst_1) {
                    return $var389[1].Emitter.Emit(inst_1.head);
                });
            }

        case 1:
            return null;
    }
}

function evaluateEntity(e) {
    var $var390 = e.Kind.Case === "Variable" ? [4, e.Kind.Fields[1]] : e.Kind.Case === "GlobalValue" ? [5, e.Kind.Fields[1], e.Kind.Fields[0]] : e.Kind.Case === "Member" ? e.Kind.Fields[1].Kind.Case === "MemberName" ? [6, e.Kind.Fields[0], e.Kind.Fields[1].Kind.Fields[0]] : [9, e.Kind.Fields[0]] : e.Kind.Case === "MemberAccess" ? [7, e.Kind.Fields[0]] : e.Kind.Case === "Call" ? e.Kind.Fields[1].Kind.Case === "ArgumentList" ? [8, e.Kind.Fields[1].Kind.Fields[0], e.Kind.Fields[0]] : [10, e.Kind.Fields[0]] : e.Kind.Case === "Operator" ? e.Kind.Fields[1].Case === "Power" ? [11, e.Kind.Fields[0], e.Kind.Fields[2]] : [12, e.Kind.Fields[0], e.Kind.Fields[1], e.Kind.Fields[2]] : e.Kind.Case === "List" ? [13, e.Kind.Fields[0]] : e.Kind.Case === "Placeholder" ? [14, e.Kind.Fields[1]] : e.Kind.Case === "ArgumentList" ? [15] : e.Kind.Case === "NamedParam" ? [15] : e.Kind.Case === "MemberName" ? [15] : e.Kind.Case === "Binding" ? [15] : e.Kind.Case === "Root" ? [15] : e.Kind.Case === "CallSite" ? [15] : e.Kind.Case === "Function" ? [16] : e.Kind.Case === "Program" ? [16] : e.Kind.Case === "LetCommand" ? [16] : e.Kind.Case === "RunCommand" ? [16] : e.Kind.Fields[0].Case === "Number" ? [1, e.Kind.Fields[0].Fields[0]] : e.Kind.Fields[0].Case === "String" ? [2, e.Kind.Fields[0].Fields[0]] : e.Kind.Fields[0].Case === "Empty" ? [3] : [0, e.Kind.Fields[0].Fields[0]];

    switch ($var390[0]) {
        case 0:
            return $var390[1];

        case 1:
            return $var390[1];

        case 2:
            return $var390[1];

        case 3:
            return null;

        case 4:
            return (0, _Util.defaultArg)($var390[1].Value, null, function (v) {
                return v.Value;
            });

        case 5:
            if ($var390[1] != null) {
                return evaluateExpression([], $var390[1]);
            } else {
                return null;
            }

        case 6:
            var matchValue = $var390[1].Type;
            var $var391 = void 0;

            if (matchValue.Case === "Object") {
                var activePatternResult7115 = function (obj) {
                    return (0, _astops.$7C$FindMember$7C$_$7C$)($var390[2], obj);
                }(matchValue.Fields[0]);

                if (activePatternResult7115 != null) {
                    $var391 = [0, activePatternResult7115];
                } else {
                    $var391 = [1];
                }
            } else {
                $var391 = [1];
            }

            switch ($var391[0]) {
                case 0:
                    return evaluateExpr((0, _List.ofArray)([getValue($var390[1])]), function (inst) {
                        return $var391[1].Emitter.Emit(inst.head);
                    });

                case 1:
                    return null;
            }

        case 7:
            return getValue($var390[1]);

        case 8:
            var pb = (0, _Seq.toList)((0, _Seq.takeWhile)(function (_arg1) {
                return _arg1.Kind.Case === "NamedParam" ? false : true;
            }, $var390[1]));
            var nb = (0, _Seq.toList)((0, _Seq.skipWhile)(function (_arg2) {
                return _arg2.Kind.Case === "NamedParam" ? false : true;
            }, $var390[1]));
            var positionBased = (0, _from2.default)((0, _List.map)(function (e_1) {
                return getValue(e_1);
            }, pb));
            var nameBased = new _map2.default((0, _List.choose)(function (_arg3) {
                if (_arg3.Kind.Case === "NamedParam") {
                    var value = _arg3.Kind.Fields[1];
                    var name = _arg3.Kind.Fields[0];
                    return [name.Name, getValue(value)];
                } else {
                    return null;
                }
            }, nb));
            var expectedArgs = void 0;
            var matchValue_1 = $var390[2].Type;

            if (matchValue_1.Case === "Method") {
                expectedArgs = matchValue_1.Fields[0];
            } else {
                expectedArgs = new _List2.default();
            }

            var pars = (0, _List.mapIndexed)(function (i, ma) {
                return i < positionBased.length ? positionBased[i] : nameBased.has(ma.Name) ? nameBased.get(ma.Name) : null;
            }, expectedArgs);
            var $var392 = $var390[2].Kind.Case === "MemberAccess" ? $var390[2].Kind.Fields[0].Kind.Case === "Member" ? $var390[2].Kind.Fields[0].Kind.Fields[1].Kind.Case === "MemberName" ? [0, $var390[2].Kind.Fields[0].Kind.Fields[0], $var390[2].Kind.Fields[0].Kind.Fields[1].Kind.Fields[0]] : [1] : [1] : [1];

            switch ($var392[0]) {
                case 0:
                    var instValue = getValue($var392[1]);
                    var matchValue_2 = $var392[1].Type;
                    var $var393 = void 0;

                    if (matchValue_2 != null) {
                        if (matchValue_2.Case === "Object") {
                            var activePatternResult7124 = function (obj_1) {
                                return (0, _astops.$7C$FindMember$7C$_$7C$)($var392[2], obj_1);
                            }(matchValue_2.Fields[0]);

                            if (activePatternResult7124 != null) {
                                $var393 = [0, activePatternResult7124];
                            } else {
                                $var393 = [1];
                            }
                        } else {
                            $var393 = [1];
                        }
                    } else {
                        $var393 = [1];
                    }

                    switch ($var393[0]) {
                        case 0:
                            return evaluateExpr(new _List2.default(instValue, pars), function (stored) {
                                return _babel.BabelOperators.op_DivideAtDivide($var393[1].Emitter.Emit(stored.head), stored.tail);
                            });

                        case 1:
                            return evaluateExpr(new _List2.default(instValue, pars), function (stored_1) {
                                return _babel.BabelOperators.op_DivideAtDivide(_babel.BabelOperators.op_DivideQmarkDivide(stored_1.head, _babel.BabelOperators.str($var392[2].Name)), stored_1.tail);
                            });
                    }

                case 1:
                    var instValue_1 = getValue($var390[2]);
                    return evaluateExpr(new _List2.default(instValue_1, pars), function (stored_2) {
                        return _babel.BabelOperators.op_DivideAtDivide(stored_2.head, stored_2.tail);
                    });
            }

        case 9:
            _common.Log.error("interpreter", "typeCheckEntity: Member access is missing member name!");

            return null;

        case 10:
            _common.Log.error("interpreter", "typeCheckEntity: Call to %s is missing argument list!", _astops.Entity$2E$get_Name.bind((0, _astops.lastChainElement)($var390[1]))());

            return null;

        case 11:
            return evaluateExpr((0, _List.ofArray)([getValue($var390[1]), getValue($var390[2])]), function (_arg4) {
                var $var394 = _arg4.tail != null ? _arg4.tail.tail != null ? _arg4.tail.tail.tail == null ? [0, _arg4.head, _arg4.tail.head] : [1] : [1] : [1];

                switch ($var394[0]) {
                    case 0:
                        return _babel.BabelOperators.op_DivideAtDivide(_babel.BabelOperators.op_Dynamic(_babel.BabelOperators.ident("Math"), "pow"), (0, _List.ofArray)([$var394[1], $var394[2]]));

                    case 1:
                        throw new Error("evaluateEntity: Expected two arguments");
                }
            });

        case 12:
            return evaluateExpr((0, _List.ofArray)([getValue($var390[1]), getValue($var390[3])]), function (_arg5) {
                var $var395 = _arg5.tail != null ? _arg5.tail.tail != null ? _arg5.tail.tail.tail == null ? [0, _arg5.head, _arg5.tail.head] : [1] : [1] : [1];

                switch ($var395[0]) {
                    case 0:
                        var op = void 0;

                        if ($var390[2].Case === "Equals") {
                            op = new _babel.BinaryOperator("BinaryEqualStrict", []);
                        } else if ($var390[2].Case === "Plus") {
                            op = new _babel.BinaryOperator("BinaryPlus", []);
                        } else if ($var390[2].Case === "Minus") {
                            op = new _babel.BinaryOperator("BinaryMinus", []);
                        } else if ($var390[2].Case === "Multiply") {
                            op = new _babel.BinaryOperator("BinaryMultiply", []);
                        } else if ($var390[2].Case === "Divide") {
                            op = new _babel.BinaryOperator("BinaryDivide", []);
                        } else if ($var390[2].Case === "GreaterThan") {
                            op = new _babel.BinaryOperator("BinaryGreater", []);
                        } else if ($var390[2].Case === "LessThan") {
                            op = new _babel.BinaryOperator("BinaryLess", []);
                        } else if ($var390[2].Case === "GreaterThanOrEqual") {
                            op = new _babel.BinaryOperator("BinaryGreaterOrEqual", []);
                        } else if ($var390[2].Case === "LessThanOrEqual") {
                            op = new _babel.BinaryOperator("BinaryLessOrEqual", []);
                        } else if ($var390[2].Case === "Power") {
                            throw new Error("evaluateEntity: Power is not a binary operation");
                        } else {
                            op = new _babel.BinaryOperator("BinaryModulus", []);
                        }

                        return new _babel.Expression("BinaryExpression", [op, $var395[1], $var395[2], null]);

                    case 1:
                        throw new Error("evaluateEntity: Expected two arguments");
                }
            });

        case 13:
            return evaluateExpr((0, _List.map)(function (e_2) {
                return getValue(e_2);
            }, $var390[1]), function (elements) {
                return new _babel.Expression("ArrayExpression", [elements, null]);
            });

        case 14:
            return getValue($var390[1]);

        case 15:
            return null;

        case 16:
            _common.Log.error("interpreter", "Cannot evaluate entity (probably not supported yet): %O", e);

            return null;
    }
}

function ensureValue(e) {
    if (function () {
        return e.Value == null;
    }(null)) {
        var matchValue = evaluateEntity(e);

        if (matchValue != null) {
            e.Value = new _ast.EntityValue(matchValue, new _Lazy2.default(function () {
                return evaluatePreview(e, matchValue);
            }));
        }
    }
}

function getValue(e) {
    if (function () {
        return e.Value == null;
    }(null)) {
        _common.Log.error("interpreter", "getValue: Value of entity %O has not been evaluated.", e);
    }

    return e.Value.Value;
}

function evaluateEntityTree(e) {
    var visited = new _map2.default();

    var loop = function loop(e_1) {
        if (!visited.has(e_1.Symbol) ? function () {
            return e_1.Value == null;
        }(null) : false) {
            visited.set(e_1.Symbol, true);

            var inputSequence = _astops.Entity$2E$get_Antecedents.bind(e_1)();

            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = (0, _getIterator3.default)(inputSequence), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var e_2 = _step.value;
                    loop(e_2);
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }

            ensureValue(e_1);
        }
    };

    loop(e);
    return e.Value;
}

function globalEntity(name, meta, typ, expr) {
    var Kind = new _ast.EntityKind("GlobalValue", [new _ast.Name(name), expr]);

    var _Symbol = (0, _symbol2.default)();

    var Type = typ;
    return new _ast.Entity(Kind, _Symbol, null, meta, Type, new _List2.default());
}

function evaluate(globals, e) {
    var res = evaluateEntityTree(e);
    return res;
}
//# sourceMappingURL=interpreter.js.map