"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CheckingContext = undefined;

var _map = require("babel-runtime/core-js/map");

var _map2 = _interopRequireDefault(_map);

var _typeof2 = require("babel-runtime/helpers/typeof");

var _typeof3 = _interopRequireDefault(_typeof2);

var _getIterator2 = require("babel-runtime/core-js/get-iterator");

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _from = require("babel-runtime/core-js/array/from");

var _from2 = _interopRequireDefault(_from);

var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require("babel-runtime/helpers/createClass");

var _createClass3 = _interopRequireDefault(_createClass2);

exports.addError = addError;
exports.inferListType = inferListType;
exports.resolveParameterType = resolveParameterType;
exports.checkMethodCallAsync = checkMethodCallAsync;
exports.getType = getType;
exports.typeCheckEntity = typeCheckEntity;
exports.evaluateDelayedType = evaluateDelayedType;
exports.typeCheckEntityAsync = typeCheckEntityAsync;
exports.collectTypeErrors = collectTypeErrors;
exports.typeCheckProgram = typeCheckProgram;

var _Symbol2 = require("fable-core/umd/Symbol");

var _Symbol3 = _interopRequireDefault(_Symbol2);

var _Util = require("fable-core/umd/Util");

var _ast = require("../ast/ast");

var _Seq = require("fable-core/umd/Seq");

var _List = require("fable-core/umd/List");

var _common = require("../../libraries/common");

var _typeops = require("../ast/typeops");

var _Map2 = require("fable-core/umd/Map");

var _errors = require("../ast/errors");

var _GenericComparer = require("fable-core/umd/GenericComparer");

var _GenericComparer2 = _interopRequireDefault(_GenericComparer);

var _astops = require("../ast/astops");

var _AsyncBuilder = require("fable-core/umd/AsyncBuilder");

var _String = require("fable-core/umd/String");

var _Array2 = require("fable-core/umd/Array");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var CheckingContext = exports.CheckingContext = function () {
  function CheckingContext(errors, globals, ranges, evaluate) {
    (0, _classCallCheck3.default)(this, CheckingContext);
    this.Errors = errors;
    this.Globals = globals;
    this.Ranges = ranges;
    this.Evaluate = evaluate;
  }

  (0, _createClass3.default)(CheckingContext, [{
    key: _Symbol3.default.reflection,
    value: function () {
      return {
        type: "TheGamma.TypeChecker.CheckingContext",
        interfaces: ["FSharpRecord"],
        properties: {
          Errors: (0, _Util.Array)((0, _Util.makeGeneric)(_ast.Error, {
            Range: _ast.Range
          })),
          Globals: (0, _Util.Interface)("System.Collections.Generic.IDictionary"),
          Ranges: (0, _Util.Interface)("System.Collections.Generic.IDictionary"),
          Evaluate: "function"
        }
      };
    }
  }]);
  return CheckingContext;
}();

(0, _Symbol2.setType)("TheGamma.TypeChecker.CheckingContext", CheckingContext);

function addError(ctx, ent, err) {
  ctx.Errors.push(err(ctx.Ranges.get(ent.Symbol)));
}

function inferListType(typs) {
  return (0, _Seq.reduce)(function (f) {
    return function (x, y) {
      return f(x) > f(y) ? x : y;
    };
  }(function (tuple) {
    return tuple[1];
  }), (0, _List.append)((0, _List.ofArray)([[new _ast.Type("Any", []), 0]]), (0, _List.map)(function (g) {
    return [g.head, g.length];
  }, function (list) {
    return _common.List.groupWith(function (t1) {
      return function (t2) {
        return (0, _typeops.typesEqual)(t1, t2);
      };
    }, list);
  }((0, _List.filter)(function (_arg1) {
    return _arg1.Case === "Any" ? false : true;
  }, typs)))))[0];
}

function resolveParameterType(instTy, parSpec) {
  if (instTy.Case === "Method") {
    var par = parSpec.Case === "Choice2Of2" ? function (source) {
      return (0, _Seq.tryItem)(parSpec.Fields[0], source);
    }(instTy.Fields[0]) : (0, _Seq.tryFind)(function (ma) {
      return ma.Name === parSpec.Fields[0];
    }, instTy.Fields[0]);

    if (par != null) {
      return par.Type;
    } else {
      throw new Error("resolveParameterType: Parameter specification was incorrect");
    }
  } else {
    throw new Error("resolveParameterType: Instance is not an object");
  }
}

function checkMethodCallAsync(methodName, ctx, parameterTypes, resultTypeFunc, argList, args) {
  return function (builder_) {
    return builder_.Delay(function () {
      var patternInput = void 0;
      var pb = (0, _Seq.toList)((0, _Seq.takeWhile)(function (_arg1) {
        return _arg1.Kind.Case === "NamedParam" ? false : true;
      }, args));
      var nb = (0, _Seq.toList)((0, _Seq.skipWhile)(function (_arg2) {
        return _arg2.Kind.Case === "NamedParam" ? false : true;
      }, args));
      patternInput = [(0, _from2.default)(pb), (0, _Map2.create)((0, _List.choose)(function (arg) {
        if (arg.Kind.Case === "NamedParam") {
          return [arg.Kind.Fields[0].Name, arg.Kind.Fields[1]];
        } else {
          (function (err) {
            addError(ctx, arg, err);
          })(function (rng) {
            return _errors.TypeChecker.nameBasedParamMustBeLast(rng);
          });

          return null;
        }
      }, nb), new _GenericComparer2.default(_Util.compare))];
      var matchedArguments = (0, _List.mapIndexed)(function (index, ma) {
        var arg_1 = index < patternInput[0].length ? patternInput[0][index] : (0, _Map2.tryFind)(ma.Name, patternInput[1]);

        if (arg_1 == null) {
          if (ma.Optional) {
            return [ma.Type, null];
          } else if (arg_1 == null) {
            (function (err_1) {
              addError(ctx, argList, err_1);
            })(function (rng_1) {
              return _errors.TypeChecker.parameterMissingValue(ma.Name, rng_1);
            });

            return [new _ast.Type("Any", []), null];
          } else {
            throw new Error("C:\\Tomas\\Public\\the-gamma\\thegamma-script\\src\\thegamma\\analyzer/typechecker.fs", 75, 12);
          }
        } else {
          return [getType(ctx, arg_1), ma.Static ? arg_1 : null];
        }
      }, parameterTypes);

      _common.Log.trace("typechecker", "Evaluating arguments of type-level method '%s'", methodName);

      return builder_.Combine(builder_.For((0, _Seq.choose)(function (tuple) {
        return tuple[1];
      }, matchedArguments), function (_arg3) {
        _arg3.Value = ctx.Evaluate(_arg3);
        return builder_.Zero();
      }), builder_.Delay(function () {
        _common.Log.trace("typechecker", "Evaluated arguments of '%s': %O", methodName, (0, _from2.default)((0, _Seq.delay)(function () {
          return (0, _Seq.map)(function (e) {
            return e.Value;
          }, (0, _Seq.choose)(function (tuple_1) {
            return tuple_1[1];
          }, matchedArguments));
        })));

        var tcargs = (0, _List.map)(function (_arg4) {
          if (_arg4[1] != null) {
            var e_1 = _arg4[1];
            return [_arg4[0], e_1.Value.Value];
          } else {
            return [_arg4[0], null];
          }
        }, matchedArguments);
        var matchValue = resultTypeFunc(tcargs);

        if (matchValue == null) {
          _common.Log.trace("typechecker", "Invalid argument type when calling '%s'. Argument types: %O", methodName, (0, _from2.default)((0, _List.map)(function ($var367) {
            return (0, _astops.formatType)($var367[0]);
          }, matchedArguments)));

          (function (err_2) {
            addError(ctx, argList, err_2);
          })(function (rng_2) {
            return _errors.TypeChecker.parameterConflict(rng_2);
          });

          return builder_.Return(new _ast.Type("Any", []));
        } else {
          return builder_.Return(matchValue);
        }
      }));
    });
  }(_AsyncBuilder.singleton);
}

function getType(ctx, e) {
  if (function () {
    return e.Type == null;
  }(null)) {
    var errorCount = ctx.Errors.length;
    e.Type = typeCheckEntity(ctx, e);
    e.Errors = (0, _Seq.toList)((0, _Seq.delay)(function () {
      return (0, _Seq.map)(function (i) {
        return ctx.Errors[i];
      }, (0, _Seq.range)(errorCount, ctx.Errors.length - 1));
    }));
  }

  return e.Type;
}

function typeCheckEntity(ctx, e) {
  if (e.Kind.Case === "Variable") {
    return getType(ctx, e.Kind.Fields[1]);
  } else if (e.Kind.Case === "Member") {
    if (e.Kind.Fields[1].Kind.Case === "MemberName") {
      var matchValue = getType(ctx, e.Kind.Fields[0]);
      var $var368 = void 0;

      if (matchValue.Case === "Any") {
        $var368 = [0];
      } else if (matchValue.Case === "Object") {
        var activePatternResult7033 = function (obj) {
          return (0, _astops.$7C$FindMember$7C$_$7C$)(e.Kind.Fields[1].Kind.Fields[0], obj);
        }(matchValue.Fields[0]);

        if (activePatternResult7033 != null) {
          $var368 = [1, activePatternResult7033];
        } else {
          $var368 = [2];
        }
      } else {
        $var368 = [2];
      }

      switch ($var368[0]) {
        case 0:
          return new _ast.Type("Any", []);

        case 1:
          e.Meta = $var368[1].Metadata;
          return $var368[1].Type;

        case 2:
          if (matchValue.Case === "Object") {
            (function (err) {
              addError(ctx, e.Kind.Fields[1], err);
            })(function () {
              var members = matchValue.Fields[0].Members;
              return function (rng) {
                return _errors.TypeChecker.memberMissing(e.Kind.Fields[1].Kind.Fields[0].Name, members, rng);
              };
            }());

            return new _ast.Type("Any", []);
          } else {
            (function (err_1) {
              addError(ctx, e.Kind.Fields[0], err_1);
            })(function (rng_1) {
              return _errors.TypeChecker.notAnObject(e.Kind.Fields[1].Kind.Fields[0].Name, matchValue, rng_1);
            });

            return new _ast.Type("Any", []);
          }

      }
    } else {
      _common.Log.error("typechecker", "typeCheckEntity: Member access is missing member name!");

      throw new Error("typeCheckEntity: Member access is missing member name!");
    }
  } else if (e.Kind.Case === "MemberAccess") {
    return getType(ctx, e.Kind.Fields[0]);
  } else if (e.Kind.Case === "Call") {
    if (e.Kind.Fields[1].Kind.Case === "ArgumentList") {
      _common.Log.error("typechecker", "typeCheckEntity: Call to %s has not been type-checked in typeCheckEntityAsync!", _astops.Entity$2E$get_Name.bind((0, _astops.lastChainElement)(e.Kind.Fields[0]))());

      return (0, _String.fsFormat)("typeCheckEntity: Call to %s has not been type-checked in typeCheckEntityAsync!")(function (x) {
        throw new Error(x);
      })(_astops.Entity$2E$get_Name.bind((0, _astops.lastChainElement)(e.Kind.Fields[0]))());
    } else {
      _common.Log.error("typechecker", "typeCheckEntity: Call to %s is missing argument list!", _astops.Entity$2E$get_Name.bind((0, _astops.lastChainElement)(e.Kind.Fields[0]))());

      return (0, _String.fsFormat)("typeCheckEntity: Call to %s is missing argument list!")(function (x) {
        throw new Error(x);
      })(_astops.Entity$2E$get_Name.bind((0, _astops.lastChainElement)(e.Kind.Fields[0]))());
    }
  } else if (e.Kind.Case === "Placeholder") {
    return getType(ctx, e.Kind.Fields[1]);
  } else if (e.Kind.Case === "Operator") {
    var operandTypes = (0, _List.map)(function (arg0) {
      return new _ast.Type("Primitive", [arg0]);
    }, e.Kind.Fields[1].Case === "Equals" ? (0, _List.ofArray)([new _ast.PrimitiveType("Number", []), new _ast.PrimitiveType("String", []), new _ast.PrimitiveType("Date", []), new _ast.PrimitiveType("Bool", [])]) : e.Kind.Fields[1].Case === "Plus" ? (0, _List.ofArray)([new _ast.PrimitiveType("Number", []), new _ast.PrimitiveType("String", [])]) : (0, _List.ofArray)([new _ast.PrimitiveType("Number", [])]));
    (0, _Seq.iterateIndexed)(function (idx, operand) {
      var typ = getType(ctx, operand);

      if ((0, _Seq.forAll)(function (opt) {
        return !(0, _typeops.typesEqual)(typ, opt);
      }, operandTypes)) {
        (function (err_2) {
          addError(ctx, operand, err_2);
        })(function (rng_2) {
          return _errors.TypeChecker.numericOperatorExpectsNumbers(e.Kind.Fields[1], idx, operandTypes, typ, rng_2);
        });
      }
    }, (0, _List.ofArray)([e.Kind.Fields[0], e.Kind.Fields[2]]));
    var $var369 = e.Kind.Fields[1].Case === "Equals" ? [0] : e.Kind.Fields[1].Case === "LessThan" ? [0] : e.Kind.Fields[1].Case === "GreaterThan" ? [0] : e.Kind.Fields[1].Case === "LessThanOrEqual" ? [0] : e.Kind.Fields[1].Case === "GreaterThanOrEqual" ? [0] : [1];

    switch ($var369[0]) {
      case 0:
        return new _ast.Type("Primitive", [new _ast.PrimitiveType("Bool", [])]);

      case 1:
        return getType(ctx, e.Kind.Fields[0]);
    }
  } else if (e.Kind.Case === "List") {
    var _ret = function () {
      var typs = (0, _List.map)(function (e_1) {
        return getType(ctx, e_1);
      }, e.Kind.Fields[0]);
      var typ_1 = inferListType(typs);
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        var _loop = function _loop() {
          var a = _step.value;
          var elty = getType(ctx, a);

          if (!(0, _typeops.typesEqual)(typ_1, elty)) {
            (function (err_3) {
              addError(ctx, a, err_3);
            })(function (rng_3) {
              return _errors.TypeChecker.listElementTypeDoesNotMatch(typ_1, elty, rng_3);
            });
          }
        };

        for (var _iterator = (0, _getIterator3.default)(e.Kind.Fields[0]), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          _loop();
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return {
        v: new _ast.Type("List", [typ_1])
      };
    }();

    if ((typeof _ret === "undefined" ? "undefined" : (0, _typeof3.default)(_ret)) === "object") return _ret.v;
  } else if (e.Kind.Case === "Binding") {
    if (e.Kind.Fields[1].Kind.Case === "CallSite") {
      var matchValue_1 = resolveParameterType(getType(ctx, e.Kind.Fields[1].Kind.Fields[0]), e.Kind.Fields[1].Kind.Fields[1]);
      var $var370 = matchValue_1.Case === "Method" ? matchValue_1.Fields[0].tail != null ? matchValue_1.Fields[0].tail.tail == null ? [0, matchValue_1.Fields[0].head] : [1] : [1] : [1];

      switch ($var370[0]) {
        case 0:
          return $var370[1].Type;

        case 1:
          throw new Error("typeCheckEntity: Expected parameter of function type");
      }
    } else {
      return (0, _String.fsFormat)("typeCheckEntity: Variable binding %s is missing call site!")(function (x) {
        throw new Error(x);
      })(e.Kind.Fields[0].Name);
    }
  } else if (e.Kind.Case === "Function") {
    var resTyp = getType(ctx, e.Kind.Fields[1]);
    return new _ast.Type("Method", [(0, _List.ofArray)([new _ast.MethodArgument("", false, false, getType(ctx, e.Kind.Fields[0]))]), function (_arg1) {
      return resTyp;
    }]);
  } else if (e.Kind.Case === "Constant") {
    if (e.Kind.Fields[0].Case === "String") {
      return new _ast.Type("Primitive", [new _ast.PrimitiveType("String", [])]);
    } else if (e.Kind.Fields[0].Case === "Boolean") {
      return new _ast.Type("Primitive", [new _ast.PrimitiveType("Bool", [])]);
    } else if (e.Kind.Fields[0].Case === "Empty") {
      return new _ast.Type("Any", []);
    } else {
      return new _ast.Type("Primitive", [new _ast.PrimitiveType("Number", [])]);
    }
  } else if (e.Kind.Case === "Root") {
    return new _ast.Type("Any", []);
  } else if (e.Kind.Case === "LetCommand") {
    return new _ast.Type("Any", []);
  } else if (e.Kind.Case === "RunCommand") {
    return new _ast.Type("Any", []);
  } else if (e.Kind.Case === "ArgumentList") {
    return new _ast.Type("Any", []);
  } else if (e.Kind.Case === "NamedParam") {
    return new _ast.Type("Any", []);
  } else if (e.Kind.Case === "CallSite") {
    return new _ast.Type("Any", []);
  } else if (e.Kind.Case === "Program") {
    return new _ast.Type("Any", []);
  } else if (e.Kind.Case === "MemberName") {
    return new _ast.Type("Any", []);
  } else if (!ctx.Globals.has(e.Kind.Fields[0].Name)) {
    (function (err_4) {
      addError(ctx, e, err_4);
    })(function (rng_4) {
      return _errors.TypeChecker.variableNotInScope(e.Kind.Fields[0].Name, rng_4);
    });

    return new _ast.Type("Any", []);
  } else {
    return getType(ctx, ctx.Globals.get(e.Kind.Fields[0].Name));
  }
}

function evaluateDelayedType(topLevel, t) {
  return function (builder_) {
    return builder_.Delay(function () {
      return t.Case === "Delayed" ? builder_.Bind((0, _common.Async$2E$AwaitFuture$2E$Static)(t.Fields[0]), function (_arg1) {
        return builder_.ReturnFrom(evaluateDelayedType(topLevel, _arg1));
      }) : builder_.Return(t);
    });
  }(_AsyncBuilder.singleton);
}

function typeCheckEntityAsync(ctx, e) {
  return function (builder_) {
    return builder_.Delay(function () {
      var visited = new _map2.default();

      var loop = function loop(e_1) {
        return function (builder__1) {
          return builder__1.Delay(function () {
            var isGlobal = e_1.Kind.Case === "GlobalValue" ? true : false;

            if (!visited.has(e_1.Symbol) ? isGlobal ? true : function () {
              return e_1.Type == null;
            }(null) : false) {
              visited.set(e_1.Symbol, true);
              return builder__1.Combine(builder__1.For(_astops.Entity$2E$get_Antecedents.bind(e_1)(), function (_arg1) {
                return builder__1.Bind(loop(_arg1), function () {
                  return builder__1.Return(null);
                });
              }), builder__1.Delay(function () {
                return builder__1.Combine(function () {
                  var $var371 = e_1.Kind.Case === "Call" ? e_1.Kind.Fields[1].Kind.Case === "ArgumentList" ? [0, e_1.Kind.Fields[1], e_1.Kind.Fields[1].Kind.Fields[0], e_1.Kind.Fields[0]] : [1] : [1];

                  switch ($var371[0]) {
                    case 0:
                      var errorCount = ctx.Errors.length;
                      return builder__1.Bind(function () {
                        var matchValue = getType(ctx, $var371[3]);

                        if (matchValue.Case === "Any") {
                          return _AsyncBuilder.singleton.Return(new _ast.Type("Any", []));
                        } else if (matchValue.Case === "Method") {
                          return checkMethodCallAsync(_astops.Entity$2E$get_Name.bind($var371[3])(), ctx, matchValue.Fields[0], matchValue.Fields[1], $var371[1], $var371[2]);
                        } else {
                          var lastName = (0, _astops.lastChainElement)($var371[3]);

                          (function (err) {
                            addError(ctx, $var371[3], err);
                          })(function () {
                            var name = _astops.Entity$2E$get_Name.bind(lastName)();

                            return function (rng) {
                              return _errors.TypeChecker.notAnMethod(name, matchValue, rng);
                            };
                          }());

                          return _AsyncBuilder.singleton.Return(new _ast.Type("Any", []));
                        }
                      }(), function (_arg3) {
                        e_1.Type = _arg3;
                        e_1.Errors = (0, _Seq.toList)((0, _Seq.delay)(function () {
                          return (0, _Seq.map)(function (i) {
                            return ctx.Errors[i];
                          }, (0, _Seq.range)(errorCount, ctx.Errors.length - 1));
                        }));
                        return builder__1.Zero();
                      });

                    case 1:
                      return builder__1.Zero();
                  }
                }(), builder__1.Delay(function () {
                  return builder__1.Bind(evaluateDelayedType(true, getType(ctx, e_1)), function (_arg4) {
                    _common.Log.trace("typechecker", "Type of entity '%s' (%s) is: %s", _astops.Entity$2E$get_Name.bind(e_1)(), (0, _astops.formatEntityKind)(e_1.Kind), (0, _astops.formatType)(_arg4));

                    e_1.Type = _arg4;
                    return builder__1.Zero();
                  });
                }));
              }));
            } else {
              return builder__1.Zero();
            }
          });
        }(_AsyncBuilder.singleton);
      };

      return builder_.Bind(loop(e), function () {
        return builder_.Return(getType(ctx, e));
      });
    });
  }(_AsyncBuilder.singleton);
}

function collectTypeErrors(entity) {
  var errors = [];
  var visited = new _map2.default();

  var loop = function loop(e) {
    if (!visited.has(e.Symbol)) {
      visited.set(e.Symbol, true);

      var inputSequence = _astops.Entity$2E$get_Antecedents.bind(e)();

      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = (0, _getIterator3.default)(inputSequence), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var e_1 = _step2.value;
          loop(e_1);
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      (0, _Array2.addRangeInPlace)(e.Errors, errors);
    }
  };

  loop(entity);
  return (0, _from2.default)(errors);
}

function typeCheckProgram(globals, bound, evaluate, prog) {
  return function (builder_) {
    return builder_.Delay(function () {
      _common.Log.trace("typechecker", "Type checking program");

      return builder_.TryWith(builder_.Delay(function () {
        var rangeLookup = new _map2.default((0, _Seq.toList)((0, _Seq.delay)(function () {
          return (0, _Seq.collect)(function (matchValue) {
            return (0, _Seq.singleton)([matchValue[1].Symbol, matchValue[0]]);
          }, bound.Entities);
        })));
        var vars = new _map2.default((0, _Seq.toList)((0, _Seq.delay)(function () {
          return (0, _Seq.map)(function (e) {
            return [_astops.Entity$2E$get_Name.bind(e)(), e];
          }, globals);
        })));
        var ctx = new CheckingContext([], vars, rangeLookup, evaluate);
        return builder_.Bind(typeCheckEntityAsync(ctx, prog), function (_arg1) {
          _common.Log.trace("typechecker", "Completed type checking");

          return builder_.Zero();
        });
      }), function (_arg2) {
        _common.Log.exn("typechecker", "Type checking program failed: %O", _arg2);

        return builder_.Zero();
      });
    });
  }(_AsyncBuilder.singleton);
}
//# sourceMappingURL=typechecker.js.map