"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.providers = exports.gamma = exports.editor = exports.checkingResult = exports.thenable = exports.defaultEditorOptions = exports.error = exports.editorOptions = exports.previews = exports.evaluate = exports.TheGammaContext = exports.TheGammaProviders = undefined;

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

var _from = require("babel-runtime/core-js/array/from");

var _from2 = _interopRequireDefault(_from);

var _symbol = require("babel-runtime/core-js/symbol");

var _symbol2 = _interopRequireDefault(_symbol);

var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require("babel-runtime/helpers/createClass");

var _createClass3 = _interopRequireDefault(_createClass2);

exports.buildGlobalsTable = buildGlobalsTable;
exports.resolveProvider = resolveProvider;
exports.callShowMethod = callShowMethod;
exports.serializeType = serializeType;
exports.serializeEntity = serializeEntity;

var _common = require("../libraries/common");

var _Map = require("fable-core/umd/Map");

var _GenericComparer = require("fable-core/umd/GenericComparer");

var _GenericComparer2 = _interopRequireDefault(_GenericComparer);

var _Util = require("fable-core/umd/Util");

var _String = require("fable-core/umd/String");

var _Seq = require("fable-core/umd/Seq");

var _providers = require("./providers/providers");

var _ast = require("./ast/ast");

var _List = require("fable-core/umd/List");

var _List2 = _interopRequireDefault(_List);

var _interpreter = require("./analyzer/interpreter");

var _AsyncBuilder = require("fable-core/umd/AsyncBuilder");

var _rest = require("./providers/rest");

var _pivot = require("./providers/pivot");

var _Symbol3 = require("fable-core/umd/Symbol");

var _Symbol4 = _interopRequireDefault(_Symbol3);

var _services = require("./services");

var _astops = require("./ast/astops");

var _codegen = require("./codegen/codegen");

var _series = require("../libraries/series");

var _runtime = require("./codegen/runtime");

var _charts = require("../libraries/google/charts");

var _tables = require("../libraries/tables");

var _general = require("../libraries/general");

var _interactive = require("../libraries/interactive");

var _pivot2 = require("./live/pivot");

var _showable = require("./live/showable");

var _Choice = require("fable-core/umd/Choice");

var _Choice2 = _interopRequireDefault(_Choice);

var _Async = require("fable-core/umd/Async");

var _Observable = require("fable-core/umd/Observable");

var _blocks = require("./blocks");

var _monaco = require("./monaco");

var _fsharp = require("./providers/fsharp");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function buildGlobalsTable(provideTypes) {
  return function (arg00) {
    return function (arg10) {
      return (0, _common.Async$2E$StartAsNamedFuture$2E$Static)(arg00, arg10);
    };
  }("buildGlobalsTable")(function (builder_) {
    return builder_.Delay(function () {
      var named = (0, _Map.create)(null, new _GenericComparer2.default(_Util.compare));

      var lookupNamed = function lookupNamed(n) {
        var matchValue = (0, _Map.tryFind)(n, named);

        if (matchValue == null) {
          _common.Log.error("typechecker", "Could not find named type '%s'", n);

          throw new Error((0, _String.fsFormat)("Could not find named type '%s'")(function (x) {
            return x;
          })(n));
        } else {
          return matchValue;
        }
      };

      return builder_.Bind(provideTypes(lookupNamed), function (_arg1) {
        var allTypes = (0, _Seq.toList)((0, _Seq.delay)(function () {
          return (0, _Seq.append)((0, _Seq.singleton)(new _providers.ProvidedType("NamedType", ["value", new _ast.Type("Any", [])])), (0, _Seq.delay)(function () {
            return (0, _Seq.append)((0, _Seq.singleton)(new _providers.ProvidedType("NamedType", ["object", new _ast.Type("Any", [])])), (0, _Seq.delay)(function () {
              return (0, _Seq.append)((0, _Seq.singleton)(new _providers.ProvidedType("NamedType", ["seq", new _ast.Type("Any", [])])), (0, _Seq.delay)(function () {
                return (0, _Seq.append)((0, _Seq.singleton)(new _providers.ProvidedType("NamedType", ["async", new _ast.Type("Any", [])])), (0, _Seq.delay)(function () {
                  return _arg1;
                }));
              }));
            }));
          }));
        }));
        named = (0, _Map.create)((0, _Seq.choose)(function (_arg2) {
          return _arg2.Case === "NamedType" ? [_arg2.Fields[0], _arg2.Fields[1]] : null;
        }, allTypes), new _GenericComparer2.default(_Util.compare));
        var globalEntities = (0, _List.choose)(function (_arg3) {
          return _arg3.Case === "GlobalValue" ? (0, _interpreter.globalEntity)(_arg3.Fields[0], _arg3.Fields[1], _arg3.Fields[3], _arg3.Fields[2]) : null;
        }, allTypes);
        return builder_.Return(globalEntities);
      });
    });
  }(_AsyncBuilder.singleton));
}

function resolveProvider(lookup, ignoreFilter, kind, endpoint) {
  switch (kind) {
    case "rest":
      var matchValue = (0, _rest.provideRestType)(lookup, function (kind_1) {
        return function (endpoint_1) {
          return resolveProvider(lookup, ignoreFilter, kind_1, endpoint_1);
        };
      }, "anonymous", endpoint, "");

      if (matchValue.Case === "GlobalValue") {
        return [matchValue.Fields[3], new _ast.Emitter(function (_arg1) {
          return matchValue.Fields[2];
        })];
      } else {
        throw new Error("resolveProvider: Expected global value");
      }

    case "pivot":
      var pivotType = function (builder_) {
        return builder_.Delay(function () {
          return builder_.Bind((0, _pivot.providePivotType)(endpoint, ignoreFilter, "anonymous", lookup), function (_arg1_1) {
            return _arg1_1.Case === "GlobalValue" ? builder_.Return(_arg1_1.Fields[3]) : builder_.Return(function () {
              throw new Error("resolveProvider: Expected global value");
            }());
          });
        });
      }(_AsyncBuilder.singleton);

      return [new _ast.Type("Delayed", [function (arg00) {
        return function (arg10) {
          return (0, _common.Async$2E$StartAsNamedFuture$2E$Static)(arg00, arg10);
        };
      }("pivotType:" + endpoint)(pivotType)]), new _ast.Emitter(function (_arg2) {
        return (0, _pivot.makePivotExpression)(endpoint);
      })];

    default:
      _common.Log.error("providers", "Cannot resolve provider '%s' (%s)", kind, endpoint);

      throw new Error("resolveProvider: Cannot resolve type provider");
  }
}

var TheGammaProviders = exports.TheGammaProviders = function () {
  function TheGammaProviders(globals) {
    (0, _classCallCheck3.default)(this, TheGammaProviders);
    this.globals = globals;
  }

  (0, _createClass3.default)(TheGammaProviders, [{
    key: _Symbol4.default.reflection,
    value: function () {
      return {
        type: "TheGamma.Main.TheGammaProviders",
        interfaces: ["FSharpRecord", "System.IEquatable"],
        properties: {
          globals: (0, _Util.Interface)("TheGamma.Common.Future")
        }
      };
    }
  }, {
    key: "Equals",
    value: function (other) {
      return (0, _Util.equalsRecords)(this, other);
    }
  }]);
  return TheGammaProviders;
}();

(0, _Symbol3.setType)("TheGamma.Main.TheGammaProviders", TheGammaProviders);

var TheGammaContext = exports.TheGammaContext = function () {
  function TheGammaContext(checkingService, providers) {
    (0, _classCallCheck3.default)(this, TheGammaContext);
    this.checkingService = checkingService;
    this.providers = providers;
  }

  (0, _createClass3.default)(TheGammaContext, [{
    key: _Symbol4.default.reflection,
    value: function () {
      return {
        type: "TheGamma.Main.TheGammaContext",
        interfaces: ["FSharpRecord", "System.IEquatable"],
        properties: {
          checkingService: _services.CheckingService,
          providers: TheGammaProviders
        }
      };
    }
  }, {
    key: "Equals",
    value: function (other) {
      return (0, _Util.equalsRecords)(this, other);
    }
  }]);
  return TheGammaContext;
}();

(0, _Symbol3.setType)("TheGamma.Main.TheGammaContext", TheGammaContext);

function callShowMethod(outputId, cmd) {
  var $var578 = cmd.Node.Case === "Expr" ? cmd.Node.Fields[0].Entity != null ? cmd.Node.Fields[0].Entity.Type != null ? [0, cmd.Node.Fields[0], cmd.Node.Fields[0].Entity.Type] : [1] : [1] : [1];

  switch ($var578[0]) {
    case 0:
      if ($var578[2].Case === "Object") {
        var showTyp = (0, _Seq.tryPick)(function (_arg1) {
          var $var579 = _arg1.Name === "show" ? _arg1.Type.Case === "Method" ? _arg1.Type.Fields[0].tail != null ? _arg1.Type.Fields[0].head.Type.Case === "Primitive" ? _arg1.Type.Fields[0].head.Type.Fields[0].Case === "String" ? _arg1.Type.Fields[0].tail.tail == null ? [0, _arg1.Type] : [1] : [1] : [1] : [1] : [1] : [1];

          switch ($var579[0]) {
            case 0:
              return $var579[1];

            case 1:
              return null;
          }
        }, $var578[2].Fields[0].Members);

        if (showTyp != null) {
          var rng = new _ast.Range(cmd.Range.End, cmd.Range.End);
          var outExpr = (0, _astops.node)(rng, new _ast.Expr("String", [outputId]));
          var args = (0, _List.ofArray)([new _ast.Argument(null, outExpr)]);

          var showMember = function (node) {
            return (0, _astops.node)(rng, node);
          }(new _ast.Expr("Member", [$var578[1], (0, _astops.node)(rng, new _ast.Expr("Variable", [(0, _astops.node)(rng, new _ast.Name("show"))]))]));

          var showEntity = new _ast.Entity(new _ast.EntityKind("Root", []), (0, _symbol2.default)(), null, new _List2.default(), showTyp, new _List2.default());
          showMember.Entity = showEntity;
          var expr = (0, _astops.node)(rng, new _ast.Expr("Call", [showMember, (0, _astops.node)(rng, args)]));
          return (0, _astops.node)(cmd.Range, new _ast.Command("Expr", [expr]));
        } else {
          return cmd;
        }
      } else {
        return cmd;
      }

    case 1:
      return cmd;
  }
}

function _evaluate(ctx, code, outputId) {
  return function (builder_) {
    return builder_.Delay(function () {
      return builder_.Bind(ctx.checkingService.TypeCheck(code), function (_arg1) {
        var newBody = outputId != null ? (0, _List.map)(function (cmd) {
          return callShowMethod(outputId, cmd);
        }, _arg1[2].Body.Node) : _arg1[2].Body.Node;
        var prog = new _ast.Program(new _ast.Node(_arg1[2].Body.WhiteBefore, _arg1[2].Body.WhiteAfter, _arg1[2].Body.Range, newBody, _arg1[2].Body.Entity));
        return builder_.Bind((0, _codegen.compile)(ctx.providers.globals, code, prog), function (_arg2) {
          var s = _series.series.create(function (builder__1) {
            return builder__1.Delay(function () {
              return builder__1.Return([]);
            });
          }(_AsyncBuilder.singleton), "", "", "");

          new _runtime.RuntimeContext("lol", "", "troll");

          (function (c) {
            return function (s_1) {
              return (0, _runtime.trimLeft)(c, s_1);
            };
          });

          (function (arg00) {
            return _charts.chart.bar(arg00);
          });

          (function (f) {
            return function (g) {
              return function (data) {
                return (0, _runtime.convertTupleSequence)(f, g, data);
              };
            };
          });

          (function (f_1) {
            return function (data_1) {
              return (0, _runtime.convertSequence)(f_1, data_1);
            };
          });

          _tables.table.create(s);

          _general.date.now();

          _series.series.values(new Int32Array([1]));

          _tables.placeholder.create("");

          (function (arg00_1) {
            return _interactive.youguess.line(arg00_1);
          });

          _common.Log.trace("interpreter", "Main evaluating: %O", _arg2);

          return builder_.Return(eval(_arg2));
        });
      });
    });
  }(_AsyncBuilder.singleton);
}

exports.evaluate = _evaluate;
var previews = exports.previews = (0, _List.ofArray)([_pivot2.preview, _showable.preview]);

var editorOptions = exports.editorOptions = function () {
  function editorOptions(width, height, maxHeight, autoHeight, enablePreview, monacoOptions) {
    (0, _classCallCheck3.default)(this, editorOptions);
    this.width = width;
    this.height = height;
    this.maxHeight = maxHeight;
    this.autoHeight = autoHeight;
    this.enablePreview = enablePreview;
    this.monacoOptions = monacoOptions;
  }

  (0, _createClass3.default)(editorOptions, [{
    key: _Symbol4.default.reflection,
    value: function () {
      return {
        type: "TheGamma.Main.editorOptions",
        interfaces: ["FSharpRecord"],
        properties: {
          width: (0, _Util.Option)("number"),
          height: (0, _Util.Option)("number"),
          maxHeight: (0, _Util.Option)("number"),
          autoHeight: (0, _Util.Option)("boolean"),
          enablePreview: (0, _Util.Option)("boolean"),
          monacoOptions: (0, _Util.Option)("function")
        }
      };
    }
  }]);
  return editorOptions;
}();

(0, _Symbol3.setType)("TheGamma.Main.editorOptions", editorOptions);

var error = exports.error = function () {
  function error(number, message, startLine, startColumn, endLine, endColumn) {
    (0, _classCallCheck3.default)(this, error);
    this.number = number;
    this.message = message;
    this.startLine = startLine;
    this.startColumn = startColumn;
    this.endLine = endLine;
    this.endColumn = endColumn;
  }

  (0, _createClass3.default)(error, [{
    key: _Symbol4.default.reflection,
    value: function () {
      return {
        type: "TheGamma.Main.error",
        interfaces: ["FSharpRecord", "System.IEquatable", "System.IComparable"],
        properties: {
          number: "number",
          message: "string",
          startLine: "number",
          startColumn: "number",
          endLine: "number",
          endColumn: "number"
        }
      };
    }
  }, {
    key: "Equals",
    value: function (other) {
      return (0, _Util.equalsRecords)(this, other);
    }
  }, {
    key: "CompareTo",
    value: function (other) {
      return (0, _Util.compareRecords)(this, other);
    }
  }]);
  return error;
}();

(0, _Symbol3.setType)("TheGamma.Main.error", error);

var defaultEditorOptions = exports.defaultEditorOptions = function () {
  var width = null;
  var height = null;
  var maxHeight = null;
  var autoHeight = null;
  var monacoOptions = null;
  return new editorOptions(width, height, maxHeight, autoHeight, null, monacoOptions);
}();

var thenable = exports.thenable = function () {
  (0, _createClass3.default)(thenable, [{
    key: _Symbol4.default.reflection,
    value: function () {
      return {
        type: "TheGamma.Main.thenable",
        properties: {}
      };
    }
  }]);

  function thenable(work) {
    var _this = this;

    (0, _classCallCheck3.default)(this, thenable);
    this.resCell = new _Choice2.default("Choice1Of3", [null]);

    this.trigger = function () {};

    (function (arg00) {
      (0, _Async.startImmediate)(arg00);
    })(function (builder_) {
      return builder_.Delay(function () {
        return builder_.TryWith(builder_.Delay(function () {
          return builder_.Bind(work, function (_arg1) {
            _this.resCell = new _Choice2.default("Choice2Of3", [_arg1]);

            _this.trigger(null);

            return builder_.Zero();
          });
        }), function (_arg2) {
          _this.resCell = new _Choice2.default("Choice3Of3", [_arg2]);

          _this.trigger(null);

          return builder_.Zero();
        });
      });
    }(_AsyncBuilder.singleton));
  }

  (0, _createClass3.default)(thenable, [{
    key: "then",
    value: function (onValue, onError) {
      var _this2 = this;

      this.trigger = function () {
        var matchValue = _this2.resCell;

        if (matchValue.Case === "Choice2Of3") {
          _this2.trigger = function () {};

          onValue(matchValue.Fields[0]);
        } else if (matchValue.Case === "Choice3Of3") {
          _this2.trigger = function () {};

          if (function () {
            return onError != null;
          }(null)) {
            onError(matchValue.Fields[0]);
          }
        }
      };

      this.trigger(null);
    }
  }]);
  return thenable;
}();

(0, _Symbol3.setType)("TheGamma.Main.thenable", thenable);

function serializeType(typ) {
  if (typ.Case === "Delayed") {
    return "delayed";
  } else if (typ.Case === "Primitive") {
    if (typ.Fields[0].Case === "Bool") {
      return "bool";
    } else if (typ.Fields[0].Case === "Date") {
      return "date";
    } else if (typ.Fields[0].Case === "Number") {
      return "number";
    } else if (typ.Fields[0].Case === "String") {
      return "string";
    } else {
      return "unit";
    }
  } else if (typ.Case === "Method") {
    return {
      kind: "function",
      arguments: (0, _from2.default)((0, _List.map)(function (ma) {
        return [ma.Name, ma.Optional, ma.Static, serializeType(ma.Type)];
      }, typ.Fields[0])),
      result: serializeType(typ.Fields[1]((0, _Seq.toList)((0, _Seq.delay)(function () {
        return (0, _Seq.map)(function (ma_1) {
          return [ma_1.Type, null];
        }, typ.Fields[0]);
      }))))
    };
  } else if (typ.Case === "List") {
    return {
      kind: "array",
      type: serializeType(typ.Fields[0])
    };
  } else if (typ.Case === "Object") {
    return (0, _Util.createObj)((0, _Seq.toList)((0, _Seq.delay)(function () {
      return (0, _Seq.append)((0, _Seq.singleton)(["kind", "object"]), (0, _Seq.delay)(function () {
        return (0, _Seq.append)((0, _Util.hasInterface)(typ.Fields[0], "TheGamma.TypeProviders.FSharpProvider.GenericType") ? (0, _Seq.singleton)(["generics", (0, _from2.default)((0, _Seq.map)(function (typ_1) {
          return serializeType(typ_1);
        }, typ.Fields[0].TypeArguments))]) : function () {
          return (0, _Seq.empty)();
        }(), (0, _Seq.delay)(function () {
          return (0, _Seq.singleton)(["members", typ.Fields[0].Members.map(function (m) {
            return m.Name;
          })]);
        }));
      }));
    })));
  } else {
    return "any";
  }
}

function serializeEntity(rng, ent) {
  var patternInput = ent.Kind.Case === "Program" ? ["program", new _List2.default()] : ent.Kind.Case === "RunCommand" ? ["do", new _List2.default()] : ent.Kind.Case === "LetCommand" ? ["let", new _List2.default()] : ent.Kind.Case === "Operator" ? ["operator", new _List2.default()] : ent.Kind.Case === "List" ? ["list", new _List2.default()] : ent.Kind.Case === "Constant" ? ["constant", new _List2.default()] : ent.Kind.Case === "Function" ? ["function", new _List2.default()] : ent.Kind.Case === "GlobalValue" ? ["global", new _List2.default()] : ent.Kind.Case === "Variable" ? ["variable", new _List2.default()] : ent.Kind.Case === "Binding" ? ["binding", new _List2.default()] : ent.Kind.Case === "ArgumentList" ? ["args", new _List2.default()] : ent.Kind.Case === "CallSite" ? ["callsite", new _List2.default()] : ent.Kind.Case === "Member" ? ["member", new _List2.default()] : ent.Kind.Case === "MemberAccess" ? ["access", new _List2.default()] : ent.Kind.Case === "MemberName" ? ["name", (0, _List.ofArray)([["name", ent.Kind.Fields[0].Name]])] : ent.Kind.Case === "NamedParam" ? ["param", (0, _List.ofArray)([["name", ent.Kind.Fields[0].Name]])] : ent.Kind.Case === "Placeholder" ? ["placeholder", (0, _List.ofArray)([["name", ent.Kind.Fields[0].Name]])] : ent.Kind.Case === "Call" ? ["call", new _List2.default()] : ["root", new _List2.default()];
  return (0, _Util.createObj)((0, _Seq.toList)((0, _Seq.delay)(function () {
    return (0, _Seq.append)((0, _Seq.singleton)(["kind", patternInput[0]]), (0, _Seq.delay)(function () {
      return (0, _Seq.append)(function () {
        return rng != null;
      }(null) ? (0, _Seq.singleton)(["range", {
        start: rng.Start,
        end: rng.End
      }]) : (0, _Seq.empty)(), (0, _Seq.delay)(function () {
        return (0, _Seq.append)((0, _Seq.singleton)(["getChildren", function () {
          return (0, _from2.default)(_astops.Entity$2E$get_Antecedents.bind(ent)()).map(function () {
            var rng_1 = null;
            return function (ent_1) {
              return serializeEntity(rng_1, ent_1);
            };
          }());
        }]), (0, _Seq.delay)(function () {
          return (0, _Seq.append)((0, _Seq.singleton)(["type", function () {
            var matchValue = ent.Type;

            if (matchValue != null) {
              return serializeType(matchValue);
            } else {
              return "unknown";
            }
          }()]), (0, _Seq.delay)(function () {
            return patternInput[1];
          }));
        }));
      }));
    }));
  })));
}

var checkingResult = exports.checkingResult = function () {
  (0, _createClass3.default)(checkingResult, [{
    key: _Symbol4.default.reflection,
    value: function () {
      return {
        type: "TheGamma.Main.checkingResult",
        properties: {
          wellTyped: "boolean"
        }
      };
    }
  }]);

  function checkingResult(_wellTyped, _bindingResult, _program) {
    (0, _classCallCheck3.default)(this, checkingResult);
    this._wellTyped = _wellTyped;
    this._bindingResult = _bindingResult;
  }

  (0, _createClass3.default)(checkingResult, [{
    key: "getEntities",
    value: function () {
      return this._bindingResult.Entities.map(function (tupledArg) {
        return serializeEntity(tupledArg[0], tupledArg[1]);
      });
    }
  }, {
    key: "wellTyped",
    get: function () {
      return this._wellTyped;
    }
  }]);
  return checkingResult;
}();

(0, _Symbol3.setType)("TheGamma.Main.checkingResult", checkingResult);

var editor = exports.editor = function () {
  (0, _createClass3.default)(editor, [{
    key: _Symbol4.default.reflection,
    value: function () {
      return {
        type: "TheGamma.Main.editor",
        properties: {}
      };
    }
  }]);

  function editor(ed) {
    (0, _classCallCheck3.default)(this, editor);
    this.ed = ed;
  }

  (0, _createClass3.default)(editor, [{
    key: "getMonacoEditor",
    value: function () {
      return this.ed;
    }
  }, {
    key: "getValue",
    value: function () {
      return this.ed.getModel().getValue(1, false);
    }
  }, {
    key: "setValue",
    value: function (text) {
      this.ed.getModel().setValue(text);
    }
  }]);
  return editor;
}();

(0, _Symbol3.setType)("TheGamma.Main.editor", editor);

var gamma = exports.gamma = function () {
  (0, _createClass3.default)(gamma, [{
    key: _Symbol4.default.reflection,
    value: function () {
      return {
        type: "TheGamma.Main.gamma",
        properties: {}
      };
    }
  }]);

  function gamma(ctx) {
    (0, _classCallCheck3.default)(this, gamma);
    this.ctx = ctx;
  }

  (0, _createClass3.default)(gamma, [{
    key: "check",
    value: function (code) {
      var _this3 = this;

      return new thenable(function (builder_) {
        return builder_.Delay(function () {
          return builder_.Bind(_this3.ctx.checkingService.TypeCheck(code), function (_arg1) {
            return builder_.Return(function (tupledArg) {
              return new checkingResult(tupledArg[0], tupledArg[1], tupledArg[2]);
            }(_arg1));
          });
        });
      }(_AsyncBuilder.singleton));
    }
  }, {
    key: "evaluate",
    value: function (code, outputId) {
      var _this4 = this;

      return new thenable(function (builder_) {
        return builder_.Delay(function () {
          return builder_.TryWith(builder_.Delay(function () {
            return builder_.ReturnFrom(_evaluate(_this4.ctx, code, outputId));
          }), function (_arg2) {
            _common.Log.exn("api", "Evaluating code '%O' failed with error '%O'.", code, _arg2);

            return builder_.ReturnFrom(function () {
              throw _arg2;
            }());
          });
        });
      }(_AsyncBuilder.singleton));
    }
  }, {
    key: "errorsReported",
    value: function (f) {
      (0, _Observable.add)(function (tupledArg) {
        f((0, _from2.default)((0, _Seq.sortWith)(function (x, y) {
          return (0, _Util.compare)(function (e_1) {
            return e_1.Range.Start;
          }(x), function (e_1) {
            return e_1.Range.Start;
          }(y));
        }, tupledArg[1])).map(function (e) {
          return new error(e.Number, e.Message, e.Range.Start.Line, e.Range.Start.Column, e.Range.End.Line, e.Range.End.Column);
        }));
      }, this.ctx.checkingService.ErrorsReported);
    }
  }, {
    key: "createBlocks",
    value: function (id) {
      (0, _blocks.createBlockEditor)(this.ctx.checkingService, this.ctx.providers.globals, id);
    }
  }, {
    key: "createEditor",
    value: function (id, source, options) {
      var lineHeight = 20;
      var options_1 = options != null ? options : defaultEditorOptions;
      var el = document.getElementById(id);
      var width = (0, _Util.defaultArg)(options_1.width, el.clientWidth);
      var height = (0, _Util.defaultArg)(options_1.height, el.clientHeight);
      var maxHeight = (0, _Util.defaultArg)(options_1.maxHeight, 2147483647);
      var ed = (0, _monaco.createMonacoEditor)(id, source, this.ctx.checkingService, function (opts) {
        opts.fontSize = 15;
        opts.lineHeight = 20;
        (0, _Util.defaultArg)(options_1.monacoOptions, function (value) {
          value;
        })(opts);
        var matchValue = opts.lineHeight;

        if (matchValue != null) {
          lineHeight = matchValue;
        }
      });
      var dim = {};
      dim.width = width;
      dim.height = height;
      ed.layout(dim);
      var previewService = (0, _Util.defaultArg)(options_1.enablePreview, true) ? new _services.PreviewService(this.ctx.checkingService, this.ctx.providers.globals, ed, previews) : null;
      var lastHeight = -1;

      var autosizeEditor = function autosizeEditor() {
        var text = ed.getModel().getValue(1, false);
        var lines = 1 + (0, _String.split)(text, "\n").length;
        var zoneHeight = previewService != null ? previewService.ZoneHeight : 0;
        var height_1 = maxHeight < (200 > lines * 20 + zoneHeight ? 200 : lines * 20 + zoneHeight) ? maxHeight : 200 > lines * 20 + zoneHeight ? 200 : lines * 20 + zoneHeight;

        if (height_1 !== lastHeight) {
          lastHeight = height_1;
          var dim_1 = {};
          dim_1.width = width;
          dim_1.height = height_1;
          ed.layout(dim_1);
          el.style.height = String(dim_1.height) + "px";
          el.style.width = String(dim_1.width) + "px";
        }
      };

      if ((0, _Util.equals)(options_1.autoHeight, true)) {
        ed.getModel().onDidChangeContent(function (_arg1) {
          autosizeEditor(null);
        });

        if (previewService != null) {
          (0, _Observable.add)(function () {
            autosizeEditor(null);
          }, previewService.ZoneSizeChanged);
        }

        autosizeEditor(null);
      }

      return new editor(ed);
    }
  }], [{
    key: "createContext",
    value: function (providers) {
      var checkingSvc = new _services.CheckingService("", providers.globals);
      return new gamma(new TheGammaContext(checkingSvc, providers));
    }
  }]);
  return gamma;
}();

(0, _Symbol3.setType)("TheGamma.Main.gamma", gamma);

var providers = exports.providers = function () {
  function providers() {
    (0, _classCallCheck3.default)(this, providers);
  }

  (0, _createClass3.default)(providers, [{
    key: _Symbol4.default.reflection,
    value: function () {
      return {
        type: "TheGamma.Main.providers",
        properties: {}
      };
    }
  }], [{
    key: "createProviders",
    value: function (providers_1) {
      var globals = buildGlobalsTable(function (lookup) {
        return function (builder_) {
          return builder_.Delay(function () {
            var providers_2 = function (o) {
              return (0, _keys2.default)(o).map(function (k) {
                return {
                  "key": k,
                  "value": o[k]
                };
              });
            }(providers_1).map(function (kv) {
              return kv.value(kv.key)(lookup);
            });

            return builder_.Bind((0, _Async.parallel)(providers_2), function (_arg1) {
              return builder_.Return((0, _Seq.concat)(_arg1));
            });
          });
        }(_AsyncBuilder.singleton);
      });
      return new TheGammaProviders(globals);
    }
  }, {
    key: "rest",
    value: function (url, cookies, ignoreFilter) {
      return function (name) {
        return function (lookup) {
          return function (builder_) {
            return builder_.Delay(function () {
              var provider = (0, _rest.provideRestType)(lookup, function () {
                var ignoreFilter_1 = ignoreFilter != null ? ignoreFilter : false;
                return function (kind) {
                  return function (endpoint) {
                    return resolveProvider(lookup, ignoreFilter_1, kind, endpoint);
                  };
                };
              }(), name, url, cookies != null ? cookies : "");
              return builder_.Return((0, _List.ofArray)([provider]));
            });
          }(_AsyncBuilder.singleton);
        };
      };
    }
  }, {
    key: "library",
    value: function (url) {
      return function (_arg3) {
        return function (lookup) {
          return (0, _fsharp.provideFSharpTypes)(lookup, url);
        };
      };
    }
  }, {
    key: "pivot",
    value: function (url, ignoreFilter) {
      return function (name) {
        return function (lookup) {
          return function (builder_) {
            return builder_.Delay(function () {
              return builder_.Bind((0, _pivot.providePivotType)(url, ignoreFilter != null ? ignoreFilter : false, name, lookup), function (_arg2) {
                return builder_.Return((0, _List.ofArray)([_arg2]));
              });
            });
          }(_AsyncBuilder.singleton);
        };
      };
    }
  }]);
  return providers;
}();

(0, _Symbol3.setType)("TheGamma.Main.providers", providers);
//# sourceMappingURL=main.js.map