"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createdEditors = exports.tokensProvider = exports.noState = undefined;

var _map = require("babel-runtime/core-js/map");

var _map2 = _interopRequireDefault(_map);

var _from = require("babel-runtime/core-js/array/from");

var _from2 = _interopRequireDefault(_from);

var _defineProperty2 = require("babel-runtime/helpers/defineProperty");

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

exports.getColorClass = getColorClass;
exports.createCompletionProvider = createCompletionProvider;
exports.getService = getService;
exports.setupMonacoServices = setupMonacoServices;
exports.createMonacoEditor = createMonacoEditor;

var _Symbol2 = require("fable-core/umd/Symbol");

var _Symbol3 = _interopRequireDefault(_Symbol2);

var _tokenizer = require("./parser/tokenizer");

var _Seq = require("fable-core/umd/Seq");

var _Async = require("fable-core/umd/Async");

var _common = require("../libraries/common");

var _locations = require("./common/locations");

var _astops = require("./ast/astops");

var _List = require("fable-core/umd/List");

var _List2 = _interopRequireDefault(_List);

var _String = require("fable-core/umd/String");

var _AsyncBuilder = require("fable-core/umd/AsyncBuilder");

var _monaco = require("monaco");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var i = 1;
eval('monaco$' + i + ' = monaco');
monaco = monaco;
var noState = exports.noState = (0, _defineProperty3.default)({
  clone: function () {
    return this;
  },
  equals: function (other) {
    return true;
  }
}, _Symbol3.default.reflection, function () {
  return {
    interfaces: ["Fable.Import.monaco.languages.IState"]
  };
});

function getColorClass(_arg1) {
  var $var557 = _arg1.Case === "String" ? [0] : _arg1.Case === "QIdent" ? [1] : _arg1.Case === "Ident" ? [1] : _arg1.Case === "Dot" ? [2] : _arg1.Case === "Let" ? [3] : _arg1.Case === "Boolean" ? [3] : _arg1.Case === "Fun" ? [3] : _arg1.Case === "Arrow" ? [3] : _arg1.Case === "Number" ? [4] : [5];

  switch ($var557[0]) {
    case 0:
      return "string";

    case 1:
      return "ident";

    case 2:
      return "operator";

    case 3:
      return "keyword";

    case 4:
      return "number";

    case 5:
      return "";
  }
}

var tokensProvider = exports.tokensProvider = (0, _defineProperty3.default)({
  tokenize: function (line, state) {
    var tokens = {};
    tokens.endState = noState;
    tokens.tokens = [];
    var patternInput = (0, _tokenizer.tokenize)(line);

    for (var idx = 0; idx <= patternInput[0].length - 1; idx++) {
      var t = patternInput[0][idx];
      var tok = {};
      tok.startIndex = t.Range.Start;
      tok.scopes = getColorClass(t.Token);
      tokens.tokens.push(tok);
    }

    return tokens;
  },
  getInitialState: function () {
    return noState;
  }
}, _Symbol3.default.reflection, function () {
  return {
    interfaces: ["Fable.Import.monaco.languages.TokensProvider"]
  };
});

function createCompletionProvider(getService) {
  return (0, _defineProperty3.default)({
    get triggerCharacters() {
      return (0, _from2.default)((0, _from2.default)((0, _Seq.delay)(function () {
        return (0, _Seq.map)(function (i) {
          return String.fromCharCode(i);
        }, (0, _Seq.range)(0, 255));
      })));
    },

    provideCompletionItems: function (model, position, token) {
      return function (arg00) {
        return (0, _Async.startAsPromise)(arg00);
      }(function (builder_) {
        return builder_.Delay(function () {
          return builder_.TryWith(builder_.Delay(function () {
            var svc = getService(model.uri.toString());
            var input = model.getValue(1, false);

            _common.Log.event("editor", "completions", "", {
              source: input,
              position: position
            });

            var conv = new _locations.LocationMapper(input);
            var loc = conv.LineColToAbsolute(~~position.lineNumber, ~~position.column);
            return builder_.Bind(svc.TypeCheck(input), function (_arg1) {
              var optMembers = (0, _Seq.tryPick)(function (tupledArg) {
                var $var558 = tupledArg[1].Kind.Case === "Member" ? tupledArg[1].Kind.Fields[0].Type != null ? tupledArg[1].Kind.Fields[1].Kind.Case === "MemberName" ? function () {
                  var t = tupledArg[1].Kind.Fields[0].Type;
                  var n = tupledArg[1].Kind.Fields[1].Kind.Fields[0];

                  if (loc >= tupledArg[0].Start) {
                    return loc <= tupledArg[0].End + 1;
                  } else {
                    return false;
                  }
                }() ? [0, tupledArg[1].Kind.Fields[1].Kind.Fields[0], tupledArg[1].Kind.Fields[0].Type] : [1] : [1] : [1] : [1];

                switch ($var558[0]) {
                  case 0:
                    _common.Log.trace("completions", "Antecedant at current location (member '%s'): %O", $var558[1].Name, $var558[2]);

                    if ($var558[2].Case === "Object") {
                      return [$var558[1].Name, tupledArg[0], $var558[2].Fields[0].Members];
                    } else {
                      return null;
                    }

                  case 1:
                    var $var559 = tupledArg[1].Kind.Case === "Member" ? tupledArg[1].Kind.Fields[0].Type != null ? tupledArg[1].Kind.Fields[1].Kind.Case === "MemberName" ? [0, tupledArg[1].Kind.Fields[1].Kind.Fields[0], tupledArg[1].Kind.Fields[0].Type] : [1] : [1] : [1];

                    switch ($var559[0]) {
                      case 0:
                        _common.Log.trace("completions", "Ignoring '%s' at location %s-%s (current=%s)", $var559[1].Name, tupledArg[0].Start, tupledArg[0].End, loc);

                        return null;

                      case 1:
                        return null;
                    }

                }
              }, _arg1[1].Entities);

              var convertRange = function convertRange(rng) {
                var patternInput = conv.AbsoluteToLineCol(rng.Start);
                var patternInput_1 = conv.AbsoluteToLineCol(rng.End);
                var res = {};
                res.startColumn = patternInput[1];
                res.startLineNumber = patternInput[0];
                res.endColumn = patternInput_1[1] + 1;
                res.endLineNumber = patternInput_1[0];
                return res;
              };

              if (optMembers != null) {
                var nameRange = optMembers[1];
                var members = optMembers[2];
                var currentName = optMembers[0];
                var nameRange_1 = convertRange(nameRange);

                _common.Log.trace("completions", "providing %s members at %O", members.length, nameRange_1);

                var members_1 = members.filter(function (m) {
                  var matchValue = (0, _astops.pickMetaByType)("http://schema.thegamma.net", "CompletionItem", m.Metadata);

                  if (matchValue != null) {
                    return !matchValue["hidden"];
                  } else {
                    return true;
                  }
                });
                var completion = (0, _Seq.toList)((0, _Seq.delay)(function () {
                  return (0, _Seq.map)(function (m_1) {
                    var ci = {};
                    var k = m_1.Type.Case === "Method" ? 1 : 9;
                    ci.kind = k;
                    ci.label = m_1.Name;
                    ci.insertText = (0, _astops.escapeIdent)(m_1.Name);
                    ci.filterText = (0, _astops.escapeIdent)(currentName) + m_1.Name;

                    if (m_1.Type.Case === "Method") {
                      var patternInput_2 = function () {
                        var folder = function folder(tupledArg_1) {
                          return function (s) {
                            if (tupledArg_1[1].length > 100) {
                              return [new _List2.default(tupledArg_1[1], tupledArg_1[0]), s];
                            } else {
                              return [tupledArg_1[0], tupledArg_1[1] === "" ? s : tupledArg_1[1] + "," + s];
                            }
                          };
                        };

                        var arg = [new _List2.default(), ""];
                        return function (source) {
                          return (0, _Seq.fold)(function ($var560, $var561) {
                            return folder($var560)($var561);
                          }, [arg[0], arg[1]], source);
                        };
                      }()((0, _Seq.toList)((0, _Seq.delay)(function () {
                        return (0, _Seq.map)(function (ma) {
                          return (ma.Optional ? "?" : "") + ma.Name;
                        }, m_1.Type.Fields[0]);
                      })));

                      var args = (0, _String.join)(",\n", (0, _List.reverse)(new _List2.default(patternInput_2[1], patternInput_2[0])));
                      ci.documentation = "(" + args + ")";
                    }

                    var eo = {};
                    eo.text = (0, _astops.escapeIdent)(m_1.Name);
                    eo.range = nameRange_1;
                    ci.textEdit = eo;
                    return ci;
                  }, members_1);
                }));

                _common.Log.trace("completions", "returning %O", (0, _from2.default)(completion));

                return builder_.Return((0, _from2.default)(completion));
              } else {
                _common.Log.trace("completions", "no members at %s", loc);

                return builder_.Return((0, _from2.default)(new _List2.default()));
              }
            });
          }), function (_arg2) {
            _common.Log.exn("completions", "completions failed %O", _arg2);

            return builder_.Return([]);
          });
        });
      }(_AsyncBuilder.singleton));
    },
    resolveCompletionItem: function (item, token) {
      return item;
    }
  }, _Symbol3.default.reflection, function () {
    return {
      interfaces: ["Fable.Import.monaco.languages.CompletionItemProvider"]
    };
  });
}

var createdEditors = exports.createdEditors = new _map2.default();

function getService(uri) {
  return createdEditors.get(uri);
}

function setupMonacoServices() {
  var lang = {};
  lang.id = "thegamma";

  _monaco.languages.register(lang);

  _monaco.languages.setTokensProvider("thegamma", tokensProvider);

  _monaco.languages.registerCompletionItemProvider("thegamma", createCompletionProvider(function (uri) {
    return getService(uri);
  }));
}

function createMonacoEditor(id, code, svc, customize) {
  if (createdEditors.size === 0) {
    setupMonacoServices();
  }

  var services = {};
  var options = {};
  var scroll = {};
  scroll.vertical = "none";
  scroll.horizontal = "auto";
  options.scrollbar = scroll;
  options.value = code;
  options.language = "thegamma";
  options.lineNumbersMinChars = 3;
  options.contextmenu = false;
  options.scrollBeyondLastLine = false;
  options.overviewRulerLanes = 0;
  customize(options);

  var ed = _monaco.editor.create(document.getElementById(id), options, services);

  createdEditors.set(ed.getModel().uri.toString(), svc);
  return ed;
}
//# sourceMappingURL=monaco.js.map