"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PreviewService = exports.CheckingService = exports.LineRange = exports.Position = exports.CheckingMessage = undefined;

var _defineProperty2 = require("babel-runtime/helpers/defineProperty");

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require("babel-runtime/helpers/createClass");

var _createClass3 = _interopRequireDefault(_createClass2);

exports.offsetToLocation = offsetToLocation;
exports.rangeToLoc = rangeToLoc;

var _Symbol2 = require("fable-core/umd/Symbol");

var _Symbol3 = _interopRequireDefault(_Symbol2);

var _Util = require("fable-core/umd/Util");

var _Event2 = require("fable-core/umd/Event");

var _Event3 = _interopRequireDefault(_Event2);

var _ast = require("./ast/ast");

var _astops = require("./ast/astops");

var _List = require("fable-core/umd/List");

var _List2 = _interopRequireDefault(_List);

var _common = require("../libraries/common");

var _binder = require("./analyzer/binder");

var _AsyncBuilder = require("fable-core/umd/AsyncBuilder");

var _MailboxProcessor = require("fable-core/umd/MailboxProcessor");

var _Seq = require("fable-core/umd/Seq");

var _String = require("fable-core/umd/String");

var _parser = require("./parser/parser");

var _typechecker = require("./analyzer/typechecker");

var _interpreter = require("./analyzer/interpreter");

var _locations = require("./common/locations");

var _Async = require("fable-core/umd/Async");

var _html = require("../libraries/html");

var _virtualDom = require("virtual-dom");

var _live = require("./live/live");

var _Observable = require("fable-core/umd/Observable");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var CheckingMessage = exports.CheckingMessage = function () {
  function CheckingMessage(caseName, fields) {
    (0, _classCallCheck3.default)(this, CheckingMessage);
    this.Case = caseName;
    this.Fields = fields;
  }

  (0, _createClass3.default)(CheckingMessage, [{
    key: _Symbol3.default.reflection,
    value: function () {
      return {
        type: "TheGamma.Services.CheckingMessage",
        interfaces: ["FSharpUnion", "System.IEquatable"],
        cases: {
          IsWellTyped: ["string", _Util.Any],
          TypeCheck: ["string", _Util.Any]
        }
      };
    }
  }, {
    key: "Equals",
    value: function (other) {
      return (0, _Util.equalsUnions)(this, other);
    }
  }]);
  return CheckingMessage;
}();

(0, _Symbol2.setType)("TheGamma.Services.CheckingMessage", CheckingMessage);

var Position = exports.Position = function () {
  function Position(line, column) {
    (0, _classCallCheck3.default)(this, Position);
    this.Line = line;
    this.Column = column;
  }

  (0, _createClass3.default)(Position, [{
    key: _Symbol3.default.reflection,
    value: function () {
      return {
        type: "TheGamma.Services.Position",
        interfaces: ["FSharpRecord", "System.IEquatable", "System.IComparable"],
        properties: {
          Line: "number",
          Column: "number"
        }
      };
    }
  }, {
    key: "Equals",
    value: function (other) {
      return (0, _Util.equalsRecords)(this, other);
    }
  }, {
    key: "CompareTo",
    value: function (other) {
      return (0, _Util.compareRecords)(this, other);
    }
  }]);
  return Position;
}();

(0, _Symbol2.setType)("TheGamma.Services.Position", Position);

var LineRange = exports.LineRange = function () {
  function LineRange(start, end) {
    (0, _classCallCheck3.default)(this, LineRange);
    this.Start = start;
    this.End = end;
  }

  (0, _createClass3.default)(LineRange, [{
    key: _Symbol3.default.reflection,
    value: function () {
      return {
        type: "TheGamma.Services.LineRange",
        interfaces: ["FSharpRecord", "System.IEquatable", "System.IComparable"],
        properties: {
          Start: Position,
          End: Position
        }
      };
    }
  }, {
    key: "Equals",
    value: function (other) {
      return (0, _Util.equalsRecords)(this, other);
    }
  }, {
    key: "CompareTo",
    value: function (other) {
      return (0, _Util.compareRecords)(this, other);
    }
  }]);
  return LineRange;
}();

(0, _Symbol2.setType)("TheGamma.Services.LineRange", LineRange);

function offsetToLocation(lines, offs, lengths) {
  offsetToLocation: while (true) {
    var $var550 = lengths.tail != null ? offs <= lengths.head ? [0, lengths.head, lengths.tail] : [1] : [1];

    switch ($var550[0]) {
      case 0:
        return new Position(lines, offs);

      case 1:
        if (lengths.tail == null) {
          return new Position(lines, offs);
        } else {
          lines = lines + 1;
          offs = offs - lengths.head - 1;
          lengths = lengths.tail;
          continue offsetToLocation;
        }

    }
  }
}

function rangeToLoc(lengths, rng) {
  return new LineRange(offsetToLocation(1, rng.Start, lengths), offsetToLocation(1, rng.Start, lengths));
}

var CheckingService = exports.CheckingService = function () {
  (0, _createClass3.default)(CheckingService, [{
    key: _Symbol3.default.reflection,
    value: function () {
      return {
        type: "TheGamma.Services.CheckingService",
        properties: {
          ErrorsReported: (0, _Util.Interface)("Microsoft.FSharp.Control.IEvent")
        }
      };
    }
  }]);

  function CheckingService(article, globals) {
    var _this = this;

    (0, _classCallCheck3.default)(this, CheckingService);
    this.globals = globals;
    this.errorsReported = new _Event3.default();
    var emptyProg = new _ast.Program((0, _astops.node)(new _ast.Range(0, 0), new _List2.default()));
    this.bindingContext = (0, _common.Async$2E$StartAsFuture$2E$Static)(function (builder_) {
      return builder_.Delay(function () {
        return builder_.Bind((0, _common.Async$2E$AwaitFuture$2E$Static)(_this.globals), function (_arg1) {
          return builder_.Return((0, _binder.createContext)(_arg1, article));
        });
      });
    }(_AsyncBuilder.singleton));
    this.agent = (0, _MailboxProcessor.start)(function (inbox) {
      var loop = function loop(lastCode) {
        return function (lastResult) {
          return function (builder__1) {
            return builder__1.Delay(function () {
              return builder__1.Bind(inbox.receive(), function (_arg7) {
                if (_arg7.Case === "TypeCheck") {
                  if (_arg7.Fields[0] === lastCode) {
                    _common.Log.trace("service", "Returning previous result");

                    _arg7.Fields[1].reply(lastResult);

                    return builder__1.ReturnFrom(loop(lastCode)(lastResult));
                  } else if (_arg7.Case === "TypeCheck") {
                    _common.Log.trace("service", "Type checking source code");

                    return builder__1.Bind(_this.typeCheck(_arg7.Fields[0]), function (_arg9) {
                      if (_arg9 == null) {
                        _arg7.Fields[1].reply([false, new _binder.BindingResult([]), emptyProg]);

                        return builder__1.ReturnFrom(loop(lastCode)(lastResult));
                      } else {
                        var prog = _arg9[0];
                        var errors = _arg9[2];
                        var ents = _arg9[1];

                        _this.errorsReported.Trigger([_arg7.Fields[0], errors]);

                        var result = [errors.length === 0, ents, prog];

                        _arg7.Fields[1].reply(result);

                        return builder__1.ReturnFrom(loop(_arg7.Fields[0])(result));
                      }
                    });
                  } else {
                    throw new Error("C:\\Tomas\\Public\\the-gamma\\thegamma-script\\src\\thegamma\\services.fs", 64, 12);
                  }
                } else {
                  return builder__1.Bind(_this.typeCheck(_arg7.Fields[0]), function (_arg8) {
                    return builder__1.Combine(function () {
                      var $var551 = _arg8 != null ? function () {
                        var errs = _arg8[2];
                        return errs.length === 0;
                      }() ? [0, _arg8[2]] : [1] : [1];

                      switch ($var551[0]) {
                        case 0:
                          _arg7.Fields[1].reply(true);

                          return builder__1.Zero();

                        case 1:
                          _arg7.Fields[1].reply(false);

                          return builder__1.Zero();
                      }
                    }(), builder__1.Delay(function () {
                      return builder__1.ReturnFrom(loop(lastCode)(lastResult));
                    }));
                  });
                }
              });
            });
          }(_AsyncBuilder.singleton);
        };
      };

      return loop("")([false, new _binder.BindingResult([]), emptyProg]);
    });
  }

  (0, _createClass3.default)(CheckingService, [{
    key: "TypeCheck",
    value: function (code) {
      return this.agent.postAndAsyncReply(function (ch) {
        return new CheckingMessage("TypeCheck", [code, ch]);
      });
    }
  }, {
    key: "IsWellTyped",
    value: function (code) {
      return this.agent.postAndAsyncReply(function (ch) {
        return new CheckingMessage("IsWellTyped", [code, ch]);
      });
    }
  }, {
    key: "errorsToLineCol",
    value: function (code, errors) {
      var lengths = (0, _List.map)(function (l) {
        return l.length;
      }, (0, _Seq.toList)((0, _String.split)(code, "\n")));
      return errors.map(function (e) {
        return new _ast.Error(e.Number, e.Message, rangeToLoc(lengths, e.Range));
      });
    }
  }, {
    key: "typeCheck",
    value: function (code) {
      var _this2 = this;

      return function (builder_) {
        return builder_.Delay(function () {
          return builder_.Bind((0, _common.Async$2E$AwaitFuture$2E$Static)(_this2.globals), function (_arg2) {
            _common.Log.trace("service", "Evaluated globals");

            return builder_.Combine(builder_.For(_arg2, function (_arg3) {
              _common.Log.trace("service", "Global value '%s' : %O = %O", _astops.Entity$2E$get_Name.bind(_arg3)(), (0, _Util.defaultArg)(_arg3.Type, null, function (_arg1) {
                return (0, _astops.formatType)(_arg1);
              }), _arg3.Value);

              return builder_.Zero();
            }), builder_.Delay(function () {
              return builder_.TryWith(builder_.Delay(function () {
                var patternInput = (0, _parser.parseProgram)(code);
                return builder_.Bind((0, _common.Async$2E$AwaitFuture$2E$Static)(_this2.bindingContext), function (_arg4) {
                  var patternInput_1 = (0, _binder.bindProgram)(_arg4, patternInput[0]);
                  return builder_.Bind((0, _typechecker.typeCheckProgram)(_arg2, patternInput_1[1], function (e) {
                    return (0, _interpreter.evaluate)(_arg2, e);
                  }, patternInput_1[0]), function () {
                    var typeErrors = (0, _typechecker.collectTypeErrors)(patternInput_1[0]);

                    _common.Log.trace("service", "Type checking completed");

                    var errors_1 = void 0;
                    var errors = patternInput[1].concat(typeErrors);
                    errors_1 = _this2.errorsToLineCol(code, errors);
                    return builder_.Return([patternInput[0], patternInput_1[1], errors_1]);
                  });
                });
              }), function (_arg6) {
                _common.Log.exn("service", "Type checking failed: %O", _arg6);

                return builder_.Return(null);
              });
            }));
          });
        });
      }(_AsyncBuilder.singleton);
    }
  }, {
    key: "ErrorsReported",
    get: function () {
      return this.errorsReported.Publish;
    }
  }]);
  return CheckingService;
}();

(0, _Symbol2.setType)("TheGamma.Services.CheckingService", CheckingService);

var PreviewService = exports.PreviewService = function () {
  (0, _createClass3.default)(PreviewService, [{
    key: _Symbol3.default.reflection,
    value: function () {
      return {
        type: "TheGamma.Services.PreviewService",
        properties: {
          ZoneHeight: "number",
          ZoneSizeChanged: (0, _Util.Interface)("Microsoft.FSharp.Control.IEvent")
        }
      };
    }
  }]);

  function PreviewService(checker, globals, ed, livePreviews) {
    var _this3 = this;

    (0, _classCallCheck3.default)(this, PreviewService);
    this.checker = checker;
    this.globals = globals;
    this.ed = ed;
    this.livePreviews = livePreviews;
    this.zoneSizeChanged = new _Event3.default();
    this.currentZone = null;
    this.zoneHeight = 0;
    this.tree = {};
    this.container = document.createElement("div");
    this.lastCode = "";
    this.lastMapper = new _locations.LocationMapper("");
    this.changingEditor = false;

    var trigger = function (ed_1) {
      return _this3.createLivePreview(ed_1);
    }(this.ed);

    this.ed.onDidChangeCursorPosition(function (ce) {
      if (!_this3.changingEditor) {
        var code = _this3.ed.getModel().getValue(1, false);

        _common.Log.trace("live", "Cursor position changed: code <> lastCode = %s", code !== _this3.lastCode);

        (function (arg00) {
          (0, _Async.startImmediate)(arg00);
        })(function (builder_) {
          return builder_.Delay(function () {
            return builder_.Bind(_this3.getUpdateEventAfterChange(), function (_arg6) {
              trigger(_arg6);
              return builder_.Zero();
            });
          });
        }(_AsyncBuilder.singleton));
      }
    });
  }

  (0, _createClass3.default)(PreviewService, [{
    key: "removeZone",
    value: function () {
      var matchValue = this.currentZone;

      if (matchValue == null) {} else {
        var id = matchValue[0];
        this.ed.changeViewZones(function (accessor) {
          accessor.removeZone(id);
        });
      }

      this.currentZone = null;
      this.zoneSizeChanged.Trigger(null);
    }
  }, {
    key: "createAndAddZone",
    value: function (endLine) {
      var _this4 = this;

      var zoneId = -1;
      var zone = {};
      var node = document.createElement('div');
      node.style.width = "1000px";
      node.style.height = "1000px";
      this.container = document.createElement('div');
      this.tree = {};
      node.appendChild(this.container);
      this.ed.changeViewZones(function (accessor) {
        var matchValue = _this4.currentZone;

        if (matchValue != null) {
          var id = matchValue[0];
          accessor.removeZone(id);
        }

        zone.afterLineNumber = endLine;
        zone.heightInPx = 1;
        zone.domNode = node;
        _this4.zoneHeight = 1;
        zoneId = accessor.addZone(zone);
        _this4.currentZone = [zoneId, zone];
      });
    }
  }, {
    key: "updateZones",
    value: function (trigger, liveState) {
      var _this5 = this;

      var dom = (0, _Util.defaultArg)(liveState.CurrentPreview, null, function (p) {
        return p.Render(trigger)(liveState);
      });

      if (dom != null) {
        if (function () {
          return _this5.currentZone == null;
        }(null)) {
          var endLine = dom.Line;
          this.createAndAddZone(endLine);
        }

        var patternInput = this.currentZone;
        var newTree = (0, _html.renderVirtual)(dom.Preview);
        var patches = (0, _virtualDom.diff)(this.tree, newTree);
        this.container = (0, _virtualDom.patch)(this.container, patches);
        this.tree = newTree;

        var waitForActualHeight = function waitForActualHeight(n) {
          return function (builder_) {
            return builder_.Delay(function () {
              var newHeight = _this5.container.clientHeight;

              if (n === 10 ? true : newHeight !== 0) {
                return builder_.Return(newHeight);
              } else {
                return builder_.Bind((0, _Async.sleep)(n * n), function () {
                  return builder_.ReturnFrom(waitForActualHeight(n + 1));
                });
              }
            });
          }(_AsyncBuilder.singleton);
        };

        (function (arg00) {
          (0, _Async.startImmediate)(arg00);
        })(function (builder__1) {
          return builder__1.Delay(function () {
            return builder__1.Bind(waitForActualHeight(1), function (_arg2) {
              _common.Log.trace("live", "Old height: %s, New height: %s", _this5.zoneHeight, _arg2);

              if (_this5.zoneHeight !== _arg2 ? true : patternInput[1].afterLineNumber !== dom.Line) {
                patternInput[1].afterLineNumber = dom.Line;
                patternInput[1].heightInPx = _arg2;
                _this5.zoneHeight = _arg2;

                _this5.ed.changeViewZones(function (accessor) {
                  accessor.layoutZone(patternInput[0]);
                });

                _this5.zoneSizeChanged.Trigger(null);

                return builder__1.Zero();
              } else {
                return builder__1.Zero();
              }
            });
          });
        }(_AsyncBuilder.singleton));
      } else {
        this.removeZone();
      }
    }
  }, {
    key: "getUpdateEventAfterChange",
    value: function () {
      var _this6 = this;

      return function (builder_) {
        return builder_.Delay(function () {
          var code = _this6.ed.getModel().getValue(1, false);

          var position = _this6.ed.getPosition();

          if (code !== _this6.lastCode) {
            _this6.lastCode = code;
            _this6.lastMapper = new _locations.LocationMapper(code);

            var loc = _this6.lastMapper.LineColToAbsolute(~~position.lineNumber, ~~position.column);

            return builder_.Bind(_this6.checker.TypeCheck(code), function (_arg3) {
              return builder_.Return(new _live.LiveEvent("UpdateSource", [code, loc, _arg3[2], _this6.lastMapper]));
            });
          } else {
            var loc_1 = _this6.lastMapper.LineColToAbsolute(~~position.lineNumber, ~~position.column);

            return builder_.Return(new _live.LiveEvent("UpdateLocation", [loc_1]));
          }
        });
      }(_AsyncBuilder.singleton);
    }
  }, {
    key: "createLivePreview",
    value: function (ed) {
      var _this7 = this;

      var liveEvent = new _Event3.default();
      var noState = void 0;
      var $var552 = this;
      noState = (0, _defineProperty3.default)({}, _Symbol3.default.reflection, function () {
        return {
          interfaces: ["TheGamma.Live.Common.CustomLiveState"]
        };
      });
      var liveState = void 0;
      var Mapper = new _locations.LocationMapper("");
      var Location = 0;
      var Program = new _ast.Program((0, _astops.node)(new _ast.Range(0, 0), new _List2.default()));
      liveState = new _live.LiveState(new _List2.default(), "", Program, Mapper, Location, null, noState, null);

      var applyEvent = function applyEvent(evt) {
        var liveState_1 = (0, _live.updateLiveState)(liveState, evt);
        var newState = liveState_1.CurrentPreview == null ? null : liveState_1.CurrentPreview.Update(function (arg00) {
          liveEvent.Trigger(arg00);
        })(liveState_1)(evt);
        var newPreview = void 0;
        var $var553 = evt.Case === "UpdateSource" ? [0] : evt.Case === "UpdateLocation" ? [0] : [1];

        switch ($var553[0]) {
          case 0:
            _common.Log.trace("live", "Searching for available previews");

            var state_1 = (0, _Seq.tryPick)(function (lp) {
              var state = void 0;
              var CurrentPreview = lp;
              state = new _live.LiveState(liveState_1.Globals, liveState_1.Code, liveState_1.Program, liveState_1.Mapper, liveState_1.Location, liveState_1.Selection, lp.InitialState, CurrentPreview);
              return lp.Update(function (arg00_1) {
                liveEvent.Trigger(arg00_1);
              })(state)(evt);
            }, _this7.livePreviews);
            newPreview = state_1;
            break;

          case 1:
            newPreview = null;
            break;
        }

        var matchValue = [newState, newPreview];
        var $var554 = matchValue[0] != null ? matchValue[0].CurrentPreview != null ? matchValue[1] != null ? matchValue[1].CurrentPreview != null ? function () {
          var st_1 = matchValue[0];
          var pr_1 = matchValue[1];
          var p2 = matchValue[1].CurrentPreview;
          var p1 = matchValue[0].CurrentPreview;
          return p1.ID !== p2.ID;
        }() ? [0, matchValue[0].CurrentPreview, matchValue[1].CurrentPreview, matchValue[1], matchValue[0]] : [1] : [1] : [1] : [1] : [1];

        switch ($var554[0]) {
          case 0:
            return $var554[3];

          case 1:
            if (matchValue[0] != null) {
              var st = matchValue[0];
              return st;
            } else if (matchValue[1] != null) {
              var pr = matchValue[1];
              return pr;
            } else {
              var CurrentPreview_1 = null;
              return new _live.LiveState(liveState_1.Globals, liveState_1.Code, liveState_1.Program, liveState_1.Mapper, liveState_1.Location, liveState_1.Selection, noState, CurrentPreview_1);
            }

        }
      };

      (0, _Observable.add)(function (evt_1) {
        try {
          _common.Log.trace("live", "Updating state %O with event %O", liveState, evt_1);

          var oldState = liveState;
          liveState = applyEvent(evt_1);

          if ((evt_1.Case === "UpdateSource" ? false : true) ? oldState.Code !== liveState.Code : false) {
            _this7.changingEditor = true;
            ed.getModel().setValue(liveState.Code);
          }

          var matchValue_1 = liveState.Selection;

          if (matchValue_1 != null) {
            _this7.changingEditor = true;
            var mrng = {};
            mrng.startColumn = matchValue_1.StartColumn;
            mrng.startLineNumber = matchValue_1.StartLineNumber;
            mrng.endColumn = matchValue_1.EndColumn;
            mrng.endLineNumber = matchValue_1.EndLineNumber;
            ed.setSelection(mrng);
            var Selection = null;
            liveState = new _live.LiveState(liveState.Globals, liveState.Code, liveState.Program, liveState.Mapper, liveState.Location, Selection, liveState.State, liveState.CurrentPreview);
          }

          if (_this7.changingEditor === true) {
            _this7.changingEditor = false;

            (function (arg00_2) {
              (0, _Async.startImmediate)(arg00_2);
            })(function (builder_) {
              return builder_.Delay(function () {
                _common.Log.trace("live", "Editor changed. Getting after change event...");

                return builder_.Bind(_this7.getUpdateEventAfterChange(), function (_arg4) {
                  _common.Log.trace("live", "Editor changed. Updating state %O with event %O", liveState, _arg4);

                  liveState = applyEvent(_arg4);

                  _common.Log.trace("live", "Editor changed. New state %O", liveState);

                  var trigger = function trigger(arg00_3) {
                    liveEvent.Trigger(arg00_3);
                  };

                  _this7.updateZones(trigger, liveState);

                  return builder_.Zero();
                });
              });
            }(_AsyncBuilder.singleton));
          } else {
            var trigger_1 = function trigger_1(arg00_4) {
              liveEvent.Trigger(arg00_4);
            };

            _this7.updateZones(trigger_1, liveState);
          }
        } catch (e) {
          _common.Log.exn("live", "Error when updating state %O with event %O: %O", liveState, evt_1, e);
        }
      }, liveEvent.Publish);

      (function (arg00_5) {
        (0, _Async.startImmediate)(arg00_5);
      })(function (builder__1) {
        return builder__1.Delay(function () {
          return builder__1.Bind((0, _common.Async$2E$AwaitFuture$2E$Static)(_this7.globals), function (_arg5) {
            liveEvent.Trigger(new _live.LiveEvent("InitializeGlobals", [_arg5]));
            return builder__1.Zero();
          });
        });
      }(_AsyncBuilder.singleton));

      return function (arg00_6) {
        liveEvent.Trigger(arg00_6);
      };
    }
  }, {
    key: "ZoneSizeChanged",
    get: function () {
      return this.zoneSizeChanged.Publish;
    }
  }, {
    key: "ZoneHeight",
    get: function () {
      if (!(0, _Util.equals)(this.currentZone, null)) {
        return this.zoneHeight;
      } else {
        return 0;
      }
    }
  }]);
  return PreviewService;
}();

(0, _Symbol2.setType)("TheGamma.Services.PreviewService", PreviewService);
//# sourceMappingURL=services.js.map