import _Map from "babel-runtime/core-js/map";
import _typeof from "babel-runtime/helpers/typeof";
import _getIterator from "babel-runtime/core-js/get-iterator";
import _Array$from from "babel-runtime/core-js/array/from";
import _classCallCheck from "babel-runtime/helpers/classCallCheck";
import _createClass from "babel-runtime/helpers/createClass";
import { setType } from "fable-core/Symbol";
import _Symbol from "fable-core/Symbol";
import { compare, Interface, makeGeneric, Array as _Array } from "fable-core/Util";
import { MethodArgument, PrimitiveType, Type, Range as _Range, Error as _Error } from "../ast/ast";
import { singleton as singleton_1, collect, forAll, iterateIndexed, range, map as map_1, delay, choose as choose_1, skipWhile, takeWhile, toList, tryFind, tryItem, reduce } from "fable-core/Seq";
import { mapIndexed, choose, filter, map, ofArray, append } from "fable-core/List";
import { Async$2E$AwaitFuture$2E$Static as Async_AwaitFuture_Static, Log, List } from "../../libraries/common";
import { typesEqual } from "../ast/typeops";
import { tryFind as tryFind_1, create } from "fable-core/Map";
import { TypeChecker } from "../ast/errors";
import GenericComparer from "fable-core/GenericComparer";
import { formatEntityKind, Entity$2E$get_Antecedents as Entity_get_Antecedents, lastChainElement, Entity$2E$get_Name as Entity_get_Name, $7C$FindMember$7C$_$7C$ as _FindMember___, formatType } from "../ast/astops";
import { singleton } from "fable-core/AsyncBuilder";
import { fsFormat } from "fable-core/String";
import { addRangeInPlace } from "fable-core/Array";
export var CheckingContext = function () {
  function CheckingContext(errors, globals, ranges, evaluate) {
    _classCallCheck(this, CheckingContext);

    this.Errors = errors;
    this.Globals = globals;
    this.Ranges = ranges;
    this.Evaluate = evaluate;
  }

  _createClass(CheckingContext, [{
    key: _Symbol.reflection,
    value: function () {
      return {
        type: "TheGamma.TypeChecker.CheckingContext",
        interfaces: ["FSharpRecord"],
        properties: {
          Errors: _Array(makeGeneric(_Error, {
            Range: _Range
          })),
          Globals: Interface("System.Collections.Generic.IDictionary"),
          Ranges: Interface("System.Collections.Generic.IDictionary"),
          Evaluate: "function"
        }
      };
    }
  }]);

  return CheckingContext;
}();
setType("TheGamma.TypeChecker.CheckingContext", CheckingContext);
export function addError(ctx, ent, err) {
  ctx.Errors.push(err(ctx.Ranges.get(ent.Symbol)));
}
export function inferListType(typs) {
  return reduce(function (f) {
    return function (x, y) {
      return f(x) > f(y) ? x : y;
    };
  }(function (tuple) {
    return tuple[1];
  }), append(ofArray([[new Type("Any", []), 0]]), map(function (g) {
    return [g.head, g.length];
  }, function (list) {
    return List.groupWith(function (t1) {
      return function (t2) {
        return typesEqual(t1, t2);
      };
    }, list);
  }(filter(function (_arg1) {
    return _arg1.Case === "Any" ? false : true;
  }, typs)))))[0];
}
export function resolveParameterType(instTy, parSpec) {
  if (instTy.Case === "Method") {
    var par = parSpec.Case === "Choice2Of2" ? function (source) {
      return tryItem(parSpec.Fields[0], source);
    }(instTy.Fields[0]) : tryFind(function (ma) {
      return ma.Name === parSpec.Fields[0];
    }, instTy.Fields[0]);

    if (par != null) {
      return par.Type;
    } else {
      throw new Error("resolveParameterType: Parameter specification was incorrect");
    }
  } else {
    throw new Error("resolveParameterType: Instance is not an object");
  }
}
export function checkMethodCallAsync(methodName, ctx, parameterTypes, resultTypeFunc, argList, args) {
  return function (builder_) {
    return builder_.Delay(function () {
      var patternInput = void 0;
      var pb = toList(takeWhile(function (_arg1) {
        return _arg1.Kind.Case === "NamedParam" ? false : true;
      }, args));
      var nb = toList(skipWhile(function (_arg2) {
        return _arg2.Kind.Case === "NamedParam" ? false : true;
      }, args));
      patternInput = [_Array$from(pb), create(choose(function (arg) {
        if (arg.Kind.Case === "NamedParam") {
          return [arg.Kind.Fields[0].Name, arg.Kind.Fields[1]];
        } else {
          (function (err) {
            addError(ctx, arg, err);
          })(function (rng) {
            return TypeChecker.nameBasedParamMustBeLast(rng);
          });

          return null;
        }
      }, nb), new GenericComparer(compare))];
      var matchedArguments = mapIndexed(function (index, ma) {
        var arg_1 = index < patternInput[0].length ? patternInput[0][index] : tryFind_1(ma.Name, patternInput[1]);

        if (arg_1 == null) {
          if (ma.Optional) {
            return [ma.Type, null];
          } else if (arg_1 == null) {
            (function (err_1) {
              addError(ctx, argList, err_1);
            })(function (rng_1) {
              return TypeChecker.parameterMissingValue(ma.Name, rng_1);
            });

            return [new Type("Any", []), null];
          } else {
            throw new Error("C:\\Tomas\\Public\\the-gamma\\thegamma-script\\src\\thegamma\\analyzer/typechecker.fs", 75, 12);
          }
        } else {
          return [getType(ctx, arg_1), ma.Static ? arg_1 : null];
        }
      }, parameterTypes);
      Log.trace("typechecker", "Evaluating arguments of type-level method '%s'", methodName);
      return builder_.Combine(builder_.For(choose_1(function (tuple) {
        return tuple[1];
      }, matchedArguments), function (_arg3) {
        _arg3.Value = ctx.Evaluate(_arg3);
        return builder_.Zero();
      }), builder_.Delay(function () {
        Log.trace("typechecker", "Evaluated arguments of '%s': %O", methodName, _Array$from(delay(function () {
          return map_1(function (e) {
            return e.Value;
          }, choose_1(function (tuple_1) {
            return tuple_1[1];
          }, matchedArguments));
        })));
        var tcargs = map(function (_arg4) {
          if (_arg4[1] != null) {
            var e_1 = _arg4[1];
            return [_arg4[0], e_1.Value.Value];
          } else {
            return [_arg4[0], null];
          }
        }, matchedArguments);
        var matchValue = resultTypeFunc(tcargs);

        if (matchValue == null) {
          Log.trace("typechecker", "Invalid argument type when calling '%s'. Argument types: %O", methodName, _Array$from(map(function ($var367) {
            return formatType($var367[0]);
          }, matchedArguments)));

          (function (err_2) {
            addError(ctx, argList, err_2);
          })(function (rng_2) {
            return TypeChecker.parameterConflict(rng_2);
          });

          return builder_.Return(new Type("Any", []));
        } else {
          return builder_.Return(matchValue);
        }
      }));
    });
  }(singleton);
}
export function getType(ctx, e) {
  if (function () {
    return e.Type == null;
  }(null)) {
    var errorCount = ctx.Errors.length;
    e.Type = typeCheckEntity(ctx, e);
    e.Errors = toList(delay(function () {
      return map_1(function (i) {
        return ctx.Errors[i];
      }, range(errorCount, ctx.Errors.length - 1));
    }));
  }

  return e.Type;
}
export function typeCheckEntity(ctx, e) {
  if (e.Kind.Case === "Variable") {
    return getType(ctx, e.Kind.Fields[1]);
  } else if (e.Kind.Case === "Member") {
    if (e.Kind.Fields[1].Kind.Case === "MemberName") {
      var matchValue = getType(ctx, e.Kind.Fields[0]);
      var $var368 = void 0;

      if (matchValue.Case === "Any") {
        $var368 = [0];
      } else if (matchValue.Case === "Object") {
        var activePatternResult7033 = function (obj) {
          return _FindMember___(e.Kind.Fields[1].Kind.Fields[0], obj);
        }(matchValue.Fields[0]);

        if (activePatternResult7033 != null) {
          $var368 = [1, activePatternResult7033];
        } else {
          $var368 = [2];
        }
      } else {
        $var368 = [2];
      }

      switch ($var368[0]) {
        case 0:
          return new Type("Any", []);

        case 1:
          e.Meta = $var368[1].Metadata;
          return $var368[1].Type;

        case 2:
          if (matchValue.Case === "Object") {
            (function (err) {
              addError(ctx, e.Kind.Fields[1], err);
            })(function () {
              var members = matchValue.Fields[0].Members;
              return function (rng) {
                return TypeChecker.memberMissing(e.Kind.Fields[1].Kind.Fields[0].Name, members, rng);
              };
            }());

            return new Type("Any", []);
          } else {
            (function (err_1) {
              addError(ctx, e.Kind.Fields[0], err_1);
            })(function (rng_1) {
              return TypeChecker.notAnObject(e.Kind.Fields[1].Kind.Fields[0].Name, matchValue, rng_1);
            });

            return new Type("Any", []);
          }

      }
    } else {
      Log.error("typechecker", "typeCheckEntity: Member access is missing member name!");
      throw new Error("typeCheckEntity: Member access is missing member name!");
    }
  } else if (e.Kind.Case === "MemberAccess") {
    return getType(ctx, e.Kind.Fields[0]);
  } else if (e.Kind.Case === "Call") {
    if (e.Kind.Fields[1].Kind.Case === "ArgumentList") {
      Log.error("typechecker", "typeCheckEntity: Call to %s has not been type-checked in typeCheckEntityAsync!", Entity_get_Name.bind(lastChainElement(e.Kind.Fields[0]))());
      return fsFormat("typeCheckEntity: Call to %s has not been type-checked in typeCheckEntityAsync!")(function (x) {
        throw new Error(x);
      })(Entity_get_Name.bind(lastChainElement(e.Kind.Fields[0]))());
    } else {
      Log.error("typechecker", "typeCheckEntity: Call to %s is missing argument list!", Entity_get_Name.bind(lastChainElement(e.Kind.Fields[0]))());
      return fsFormat("typeCheckEntity: Call to %s is missing argument list!")(function (x) {
        throw new Error(x);
      })(Entity_get_Name.bind(lastChainElement(e.Kind.Fields[0]))());
    }
  } else if (e.Kind.Case === "Placeholder") {
    return getType(ctx, e.Kind.Fields[1]);
  } else if (e.Kind.Case === "Operator") {
    var operandTypes = map(function (arg0) {
      return new Type("Primitive", [arg0]);
    }, e.Kind.Fields[1].Case === "Equals" ? ofArray([new PrimitiveType("Number", []), new PrimitiveType("String", []), new PrimitiveType("Date", []), new PrimitiveType("Bool", [])]) : e.Kind.Fields[1].Case === "Plus" ? ofArray([new PrimitiveType("Number", []), new PrimitiveType("String", [])]) : ofArray([new PrimitiveType("Number", [])]));
    iterateIndexed(function (idx, operand) {
      var typ = getType(ctx, operand);

      if (forAll(function (opt) {
        return !typesEqual(typ, opt);
      }, operandTypes)) {
        (function (err_2) {
          addError(ctx, operand, err_2);
        })(function (rng_2) {
          return TypeChecker.numericOperatorExpectsNumbers(e.Kind.Fields[1], idx, operandTypes, typ, rng_2);
        });
      }
    }, ofArray([e.Kind.Fields[0], e.Kind.Fields[2]]));
    var $var369 = e.Kind.Fields[1].Case === "Equals" ? [0] : e.Kind.Fields[1].Case === "LessThan" ? [0] : e.Kind.Fields[1].Case === "GreaterThan" ? [0] : e.Kind.Fields[1].Case === "LessThanOrEqual" ? [0] : e.Kind.Fields[1].Case === "GreaterThanOrEqual" ? [0] : [1];

    switch ($var369[0]) {
      case 0:
        return new Type("Primitive", [new PrimitiveType("Bool", [])]);

      case 1:
        return getType(ctx, e.Kind.Fields[0]);
    }
  } else if (e.Kind.Case === "List") {
    var _ret = function () {
      var typs = map(function (e_1) {
        return getType(ctx, e_1);
      }, e.Kind.Fields[0]);
      var typ_1 = inferListType(typs);
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        var _loop = function _loop() {
          var a = _step.value;
          var elty = getType(ctx, a);

          if (!typesEqual(typ_1, elty)) {
            (function (err_3) {
              addError(ctx, a, err_3);
            })(function (rng_3) {
              return TypeChecker.listElementTypeDoesNotMatch(typ_1, elty, rng_3);
            });
          }
        };

        for (var _iterator = _getIterator(e.Kind.Fields[0]), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          _loop();
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return {
        v: new Type("List", [typ_1])
      };
    }();

    if ((typeof _ret === "undefined" ? "undefined" : _typeof(_ret)) === "object") return _ret.v;
  } else if (e.Kind.Case === "Binding") {
    if (e.Kind.Fields[1].Kind.Case === "CallSite") {
      var matchValue_1 = resolveParameterType(getType(ctx, e.Kind.Fields[1].Kind.Fields[0]), e.Kind.Fields[1].Kind.Fields[1]);
      var $var370 = matchValue_1.Case === "Method" ? matchValue_1.Fields[0].tail != null ? matchValue_1.Fields[0].tail.tail == null ? [0, matchValue_1.Fields[0].head] : [1] : [1] : [1];

      switch ($var370[0]) {
        case 0:
          return $var370[1].Type;

        case 1:
          throw new Error("typeCheckEntity: Expected parameter of function type");
      }
    } else {
      return fsFormat("typeCheckEntity: Variable binding %s is missing call site!")(function (x) {
        throw new Error(x);
      })(e.Kind.Fields[0].Name);
    }
  } else if (e.Kind.Case === "Function") {
    var resTyp = getType(ctx, e.Kind.Fields[1]);
    return new Type("Method", [ofArray([new MethodArgument("", false, false, getType(ctx, e.Kind.Fields[0]))]), function (_arg1) {
      return resTyp;
    }]);
  } else if (e.Kind.Case === "Constant") {
    if (e.Kind.Fields[0].Case === "String") {
      return new Type("Primitive", [new PrimitiveType("String", [])]);
    } else if (e.Kind.Fields[0].Case === "Boolean") {
      return new Type("Primitive", [new PrimitiveType("Bool", [])]);
    } else if (e.Kind.Fields[0].Case === "Empty") {
      return new Type("Any", []);
    } else {
      return new Type("Primitive", [new PrimitiveType("Number", [])]);
    }
  } else if (e.Kind.Case === "Root") {
    return new Type("Any", []);
  } else if (e.Kind.Case === "LetCommand") {
    return new Type("Any", []);
  } else if (e.Kind.Case === "RunCommand") {
    return new Type("Any", []);
  } else if (e.Kind.Case === "ArgumentList") {
    return new Type("Any", []);
  } else if (e.Kind.Case === "NamedParam") {
    return new Type("Any", []);
  } else if (e.Kind.Case === "CallSite") {
    return new Type("Any", []);
  } else if (e.Kind.Case === "Program") {
    return new Type("Any", []);
  } else if (e.Kind.Case === "MemberName") {
    return new Type("Any", []);
  } else if (!ctx.Globals.has(e.Kind.Fields[0].Name)) {
    (function (err_4) {
      addError(ctx, e, err_4);
    })(function (rng_4) {
      return TypeChecker.variableNotInScope(e.Kind.Fields[0].Name, rng_4);
    });

    return new Type("Any", []);
  } else {
    return getType(ctx, ctx.Globals.get(e.Kind.Fields[0].Name));
  }
}
export function evaluateDelayedType(topLevel, t) {
  return function (builder_) {
    return builder_.Delay(function () {
      return t.Case === "Delayed" ? builder_.Bind(Async_AwaitFuture_Static(t.Fields[0]), function (_arg1) {
        return builder_.ReturnFrom(evaluateDelayedType(topLevel, _arg1));
      }) : builder_.Return(t);
    });
  }(singleton);
}
export function typeCheckEntityAsync(ctx, e) {
  return function (builder_) {
    return builder_.Delay(function () {
      var visited = new _Map();

      var loop = function loop(e_1) {
        return function (builder__1) {
          return builder__1.Delay(function () {
            var isGlobal = e_1.Kind.Case === "GlobalValue" ? true : false;

            if (!visited.has(e_1.Symbol) ? isGlobal ? true : function () {
              return e_1.Type == null;
            }(null) : false) {
              visited.set(e_1.Symbol, true);
              return builder__1.Combine(builder__1.For(Entity_get_Antecedents.bind(e_1)(), function (_arg1) {
                return builder__1.Bind(loop(_arg1), function () {
                  return builder__1.Return(null);
                });
              }), builder__1.Delay(function () {
                return builder__1.Combine(function () {
                  var $var371 = e_1.Kind.Case === "Call" ? e_1.Kind.Fields[1].Kind.Case === "ArgumentList" ? [0, e_1.Kind.Fields[1], e_1.Kind.Fields[1].Kind.Fields[0], e_1.Kind.Fields[0]] : [1] : [1];

                  switch ($var371[0]) {
                    case 0:
                      var errorCount = ctx.Errors.length;
                      return builder__1.Bind(function () {
                        var matchValue = getType(ctx, $var371[3]);

                        if (matchValue.Case === "Any") {
                          return singleton.Return(new Type("Any", []));
                        } else if (matchValue.Case === "Method") {
                          return checkMethodCallAsync(Entity_get_Name.bind($var371[3])(), ctx, matchValue.Fields[0], matchValue.Fields[1], $var371[1], $var371[2]);
                        } else {
                          var lastName = lastChainElement($var371[3]);

                          (function (err) {
                            addError(ctx, $var371[3], err);
                          })(function () {
                            var name = Entity_get_Name.bind(lastName)();
                            return function (rng) {
                              return TypeChecker.notAnMethod(name, matchValue, rng);
                            };
                          }());

                          return singleton.Return(new Type("Any", []));
                        }
                      }(), function (_arg3) {
                        e_1.Type = _arg3;
                        e_1.Errors = toList(delay(function () {
                          return map_1(function (i) {
                            return ctx.Errors[i];
                          }, range(errorCount, ctx.Errors.length - 1));
                        }));
                        return builder__1.Zero();
                      });

                    case 1:
                      return builder__1.Zero();
                  }
                }(), builder__1.Delay(function () {
                  return builder__1.Bind(evaluateDelayedType(true, getType(ctx, e_1)), function (_arg4) {
                    Log.trace("typechecker", "Type of entity '%s' (%s) is: %s", Entity_get_Name.bind(e_1)(), formatEntityKind(e_1.Kind), formatType(_arg4));
                    e_1.Type = _arg4;
                    return builder__1.Zero();
                  });
                }));
              }));
            } else {
              return builder__1.Zero();
            }
          });
        }(singleton);
      };

      return builder_.Bind(loop(e), function () {
        return builder_.Return(getType(ctx, e));
      });
    });
  }(singleton);
}
export function collectTypeErrors(entity) {
  var errors = [];
  var visited = new _Map();

  var loop = function loop(e) {
    if (!visited.has(e.Symbol)) {
      visited.set(e.Symbol, true);
      var inputSequence = Entity_get_Antecedents.bind(e)();
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = _getIterator(inputSequence), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var e_1 = _step2.value;
          loop(e_1);
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      addRangeInPlace(e.Errors, errors);
    }
  };

  loop(entity);
  return _Array$from(errors);
}
export function typeCheckProgram(globals, bound, evaluate, prog) {
  return function (builder_) {
    return builder_.Delay(function () {
      Log.trace("typechecker", "Type checking program");
      return builder_.TryWith(builder_.Delay(function () {
        var rangeLookup = new _Map(toList(delay(function () {
          return collect(function (matchValue) {
            return singleton_1([matchValue[1].Symbol, matchValue[0]]);
          }, bound.Entities);
        })));
        var vars = new _Map(toList(delay(function () {
          return map_1(function (e) {
            return [Entity_get_Name.bind(e)(), e];
          }, globals);
        })));
        var ctx = new CheckingContext([], vars, rangeLookup, evaluate);
        return builder_.Bind(typeCheckEntityAsync(ctx, prog), function (_arg1) {
          Log.trace("typechecker", "Completed type checking");
          return builder_.Zero();
        });
      }), function (_arg2) {
        Log.exn("typechecker", "Type checking program failed: %O", _arg2);
        return builder_.Zero();
      });
    });
  }(singleton);
}
//# sourceMappingURL=typechecker.js.map