import _getIterator from "babel-runtime/core-js/get-iterator";
import _classCallCheck from "babel-runtime/helpers/classCallCheck";
import _createClass from "babel-runtime/helpers/createClass";
import { choose, ofArray, append } from "fable-core/List";
import List from "fable-core/List";
import { Argument, Expr, TokenKind, Range as _Range, Node as _Node_1 } from "./ast";
import { join, fsFormat, replace } from "fable-core/String";
import { tryPick, truncate, delay, toList, iterateIndexed, map } from "fable-core/Seq";
import { setType } from "fable-core/Symbol";
import _Symbol from "fable-core/Symbol";
import { equalsRecords, Array as _Array } from "fable-core/Util";
import Choice from "fable-core/Choice";
export function node(rng, node_1) {
  var Entity = null;
  var WhiteBefore = new List();
  var WhiteAfter = new List();
  return new _Node_1(WhiteBefore, WhiteAfter, rng, node_1, Entity);
}
export function needsEscaping(s) {
  if (s[0] >= "0" ? s[0] <= "9" : false) {
    return true;
  } else {
    return s.split("").some(function (c) {
      return !(((c >= "a" ? c <= "z" : false) ? true : c >= "A" ? c <= "Z" : false) ? true : c >= "0" ? c <= "9" : false);
    });
  }
}
export function escapeIdent(s) {
  if (s === "") {
    return "''";
  } else if (needsEscaping(s)) {
    return "'" + s + "'";
  } else {
    return s;
  }
}
export function unionRanges(r1, r2) {
  return new _Range(r1.Start < r2.Start ? r1.Start : r2.Start, r1.End > r2.End ? r1.End : r2.End);
}
export function strictSubRange(first, second) {
  if (first.Start > second.Start ? first.End <= second.End : false) {
    return true;
  } else if (first.Start >= second.Start) {
    return first.End < second.End;
  } else {
    return false;
  }
}
export function formatToken(_arg1) {
  if (_arg1.Case === "RParen") {
    return ")";
  } else if (_arg1.Case === "Equals") {
    return "=";
  } else if (_arg1.Case === "Dot") {
    return ".";
  } else if (_arg1.Case === "Comma") {
    return ",";
  } else if (_arg1.Case === "Colon") {
    return ":";
  } else if (_arg1.Case === "Let") {
    return "let";
  } else if (_arg1.Case === "LSquare") {
    return "[";
  } else if (_arg1.Case === "RSquare") {
    return "]";
  } else if (_arg1.Case === "Fun") {
    return "fun";
  } else if (_arg1.Case === "Arrow") {
    return "->";
  } else if (_arg1.Case === "Operator") {
    if (_arg1.Fields[0].Case === "Divide") {
      return "/";
    } else if (_arg1.Fields[0].Case === "GreaterThan") {
      return ">";
    } else if (_arg1.Fields[0].Case === "GreaterThanOrEqual") {
      return ">=";
    } else if (_arg1.Fields[0].Case === "LessThan") {
      return "<";
    } else if (_arg1.Fields[0].Case === "LessThanOrEqual") {
      return "<=";
    } else if (_arg1.Fields[0].Case === "Minus") {
      return "-";
    } else if (_arg1.Fields[0].Case === "Multiply") {
      return "*";
    } else if (_arg1.Fields[0].Case === "Plus") {
      return "+";
    } else if (_arg1.Fields[0].Case === "Power") {
      return "^";
    } else if (_arg1.Fields[0].Case === "Equals") {
      return "=";
    } else {
      return "%";
    }
  } else if (_arg1.Case === "Boolean") {
    if (_arg1.Fields[0]) {
      return "true";
    } else {
      return "false";
    }
  } else if (_arg1.Case === "Number") {
    return _arg1.Fields[0];
  } else if (_arg1.Case === "String") {
    return "\"" + replace(replace(replace(_arg1.Fields[0], "\\", "\\\\"), "\n", "\\n"), "\"", "\\\"") + "\"";
  } else if (_arg1.Case === "Ident") {
    return _arg1.Fields[0];
  } else if (_arg1.Case === "QIdent") {
    return "'" + _arg1.Fields[0] + "'";
  } else if (_arg1.Case === "White") {
    return _arg1.Fields[0];
  } else if (_arg1.Case === "Newline") {
    return "\n";
  } else if (_arg1.Case === "Error") {
    return _arg1.Fields[0];
  } else if (_arg1.Case === "EndOfFile") {
    return "";
  } else {
    return "(";
  }
}
export function formatTokenInfo(_arg1) {
  if (_arg1.Case === "RParen") {
    return "right parenthesis `)`";
  } else if (_arg1.Case === "Equals") {
    return "equals sign `=`";
  } else if (_arg1.Case === "Dot") {
    return "dot character `.`";
  } else if (_arg1.Case === "Comma") {
    return "comma character `,`";
  } else if (_arg1.Case === "Colon") {
    return "colon character `:`";
  } else if (_arg1.Case === "Let") {
    return "`let` keyword";
  } else if (_arg1.Case === "LSquare") {
    return "left square bracket `[`";
  } else if (_arg1.Case === "RSquare") {
    return "right square bracket `]`";
  } else if (_arg1.Case === "Fun") {
    return "`fun` keyword";
  } else if (_arg1.Case === "Arrow") {
    return "arrow sign `->`";
  } else if (_arg1.Case === "Operator") {
    if (_arg1.Fields[0].Case === "Divide") {
      return "division sign `/`";
    } else if (_arg1.Fields[0].Case === "Modulo") {
      return "modulo operator `%`";
    } else if (_arg1.Fields[0].Case === "GreaterThan") {
      return "greater than sign `>`";
    } else if (_arg1.Fields[0].Case === "GreaterThanOrEqual") {
      return "greater than or equals sign `>=`";
    } else if (_arg1.Fields[0].Case === "LessThan") {
      return "less than sign `<`";
    } else if (_arg1.Fields[0].Case === "LessThanOrEqual") {
      return "less than or equals sign `<=`";
    } else if (_arg1.Fields[0].Case === "Minus") {
      return "minus sign `-`";
    } else if (_arg1.Fields[0].Case === "Multiply") {
      return "multiplication sign `*`";
    } else if (_arg1.Fields[0].Case === "Plus") {
      return "plus sign `+`";
    } else if (_arg1.Fields[0].Case === "Power") {
      return "exponentiation sign `^`";
    } else {
      return "equals operator `=`";
    }
  } else if (_arg1.Case === "Boolean") {
    if (_arg1.Fields[0]) {
      return "logical `true` value";
    } else {
      return "logical `false` value";
    }
  } else if (_arg1.Case === "Number") {
    return fsFormat("numerical value `%s`")(function (x) {
      return x;
    })(_arg1.Fields[0]);
  } else if (_arg1.Case === "String") {
    return fsFormat("string value `%s`")(function (x) {
      return x;
    })(replace(_arg1.Fields[0], "`", "'"));
  } else if (_arg1.Case === "Ident") {
    return fsFormat("identifer `%s`")(function (x) {
      return x;
    })(_arg1.Fields[0]);
  } else if (_arg1.Case === "QIdent") {
    return fsFormat("quoted identifer `'%s'`")(function (x) {
      return x;
    })(_arg1.Fields[0]);
  } else if (_arg1.Case === "White") {
    return "whitespace";
  } else if (_arg1.Case === "Newline") {
    return "end of line";
  } else if (_arg1.Case === "Error") {
    if (_arg1.Fields[0] === "`") {
      return "back-tick character";
    } else {
      return fsFormat("other character `%s`")(function (x) {
        return x;
      })(_arg1.Fields[0]);
    }
  } else if (_arg1.Case === "EndOfFile") {
    return "end of file";
  } else {
    return "left parenthesis `(`";
  }
}
export function formatTokens(tokens) {
  return join("", map(function (t) {
    return formatToken(t.Token);
  }, tokens));
}
export var FormattingContext = function () {
  function FormattingContext(strings) {
    _classCallCheck(this, FormattingContext);

    this.Strings = strings;
  }

  _createClass(FormattingContext, [{
    key: _Symbol.reflection,
    value: function () {
      return {
        type: "TheGamma.Ast.FormattingContext",
        interfaces: ["FSharpRecord", "System.IEquatable"],
        properties: {
          Strings: _Array("string")
        }
      };
    }
  }, {
    key: "Equals",
    value: function (other) {
      return equalsRecords(this, other);
    }
  }, {
    key: "Add",
    value: function (tok) {
      this.Strings.push(formatToken(tok));
    }
  }]);

  return FormattingContext;
}();
setType("TheGamma.Ast.FormattingContext", FormattingContext);
export function formatNode(ctx, f, node_1) {
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = _getIterator(node_1.WhiteBefore), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var t = _step.value;
      ctx.Add(t.Token);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  f(ctx)(node_1.Node);
  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = _getIterator(node_1.WhiteAfter), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var t_1 = _step2.value;
      ctx.Add(t_1.Token);
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }
}
export function formatName(ctx, name) {
  if (name.Name === "") {} else if (needsEscaping(name.Name)) {
    ctx.Add(new TokenKind("QIdent", [name.Name]));
  } else {
    ctx.Add(new TokenKind("Ident", [name.Name]));
  }
}
export function formatArgument(ctx, arg) {
  if (arg.Name != null) {
    formatNode(ctx, function (ctx_1) {
      return function (name) {
        formatName(ctx_1, name);
      };
    }, arg.Name);
    ctx.Add(new TokenKind("Equals", []));
  }

  formatNode(ctx, function (ctx_2) {
    return function (expr) {
      formatExpression(ctx_2, expr);
    };
  }, arg.Value);
}
export function formatExpression(ctx, expr) {
  if (expr.Case === "Member") {
    formatNode(ctx, function (ctx_1) {
      return function (expr_1) {
        formatExpression(ctx_1, expr_1);
      };
    }, expr.Fields[0]);
    ctx.Add(new TokenKind("Dot", []));
    formatNode(ctx, function (ctx_2) {
      return function (expr_2) {
        formatExpression(ctx_2, expr_2);
      };
    }, expr.Fields[1]);
  } else if (expr.Case === "Call") {
    formatNode(ctx, function (ctx_3) {
      return function (expr_3) {
        formatExpression(ctx_3, expr_3);
      };
    }, expr.Fields[0]);
    ctx.Add(new TokenKind("LParen", []));
    (function () {
      var f = function f(ctx_4) {
        return function (args) {
          iterateIndexed(function (i, arg) {
            if (i !== 0) {
              ctx_4.Add(new TokenKind("Comma", []));
            }

            formatArgument(ctx_4, arg);
          }, args);
        };
      };

      return function (node_1) {
        formatNode(ctx, f, node_1);
      };
    })()(expr.Fields[1]);
    ctx.Add(new TokenKind("RParen", []));
  } else if (expr.Case === "String") {
    ctx.Add(new TokenKind("String", [expr.Fields[0]]));
  } else if (expr.Case === "Number") {
    ctx.Add(new TokenKind("Number", [String(expr.Fields[0]), expr.Fields[0]]));
  } else if (expr.Case === "Boolean") {
    ctx.Add(new TokenKind("Boolean", [expr.Fields[0]]));
  } else if (expr.Case === "Binary") {
    formatNode(ctx, function (ctx_5) {
      return function (expr_4) {
        formatExpression(ctx_5, expr_4);
      };
    }, expr.Fields[0]);
    (function () {
      var f_1 = function f_1(ctx_6) {
        return function (op) {
          ctx_6.Add(new TokenKind("Operator", [op]));
        };
      };

      return function (node_2) {
        formatNode(ctx, f_1, node_2);
      };
    })()(expr.Fields[1]);
    formatNode(ctx, function (ctx_7) {
      return function (expr_5) {
        formatExpression(ctx_7, expr_5);
      };
    }, expr.Fields[2]);
  } else if (expr.Case === "Function") {
    ctx.Add(new TokenKind("Fun", []));
    formatNode(ctx, function (ctx_8) {
      return function (name) {
        formatName(ctx_8, name);
      };
    }, expr.Fields[0]);
    ctx.Add(new TokenKind("Arrow", []));
    formatNode(ctx, function (ctx_9) {
      return function (expr_6) {
        formatExpression(ctx_9, expr_6);
      };
    }, expr.Fields[1]);
  } else if (expr.Case === "Placeholder") {
    ctx.Add(new TokenKind("LSquare", []));
    formatNode(ctx, function (ctx_10) {
      return function (name_1) {
        formatName(ctx_10, name_1);
      };
    }, expr.Fields[0]);
    ctx.Add(new TokenKind("Colon", []));
    formatNode(ctx, function (ctx_11) {
      return function (expr_7) {
        formatExpression(ctx_11, expr_7);
      };
    }, expr.Fields[1]);
    ctx.Add(new TokenKind("RSquare", []));
  } else if (expr.Case === "List") {
    ctx.Add(new TokenKind("LSquare", []));
    var _iteratorNormalCompletion3 = true;
    var _didIteratorError3 = false;
    var _iteratorError3 = undefined;

    try {
      for (var _iterator3 = _getIterator(expr.Fields[0]), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
        var e = _step3.value;
        formatNode(ctx, function (ctx_12) {
          return function (expr_8) {
            formatExpression(ctx_12, expr_8);
          };
        }, e);
      }
    } catch (err) {
      _didIteratorError3 = true;
      _iteratorError3 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion3 && _iterator3.return) {
          _iterator3.return();
        }
      } finally {
        if (_didIteratorError3) {
          throw _iteratorError3;
        }
      }
    }

    ctx.Add(new TokenKind("RSquare", []));
  } else if (expr.Case === "Empty") {} else {
    formatNode(ctx, function (ctx_13) {
      return function (name_2) {
        formatName(ctx_13, name_2);
      };
    }, expr.Fields[0]);
  }
}
export function formatCommand(ctx, cmd) {
  if (cmd.Case === "Let") {
    ctx.Add(new TokenKind("Let", []));
    formatNode(ctx, function (ctx_1) {
      return function (name) {
        formatName(ctx_1, name);
      };
    }, cmd.Fields[0]);
    ctx.Add(new TokenKind("Equals", []));
    formatNode(ctx, function (ctx_2) {
      return function (expr) {
        formatExpression(ctx_2, expr);
      };
    }, cmd.Fields[1]);
  } else {
    formatNode(ctx, function (ctx_3) {
      return function (expr_1) {
        formatExpression(ctx_3, expr_1);
      };
    }, cmd.Fields[0]);
  }
}
export function formatSingleExpression(expr) {
  var ctx = new FormattingContext([]);
  formatNode(ctx, function (ctx_1) {
    return function (expr_1) {
      formatExpression(ctx_1, expr_1);
    };
  }, expr);
  return join("", ctx.Strings);
}
export function formatProgram(prog) {
  var ctx = new FormattingContext([]);
  (function () {
    var f = function f(ctx_1) {
      return function (cmds) {
        var _iteratorNormalCompletion4 = true;
        var _didIteratorError4 = false;
        var _iteratorError4 = undefined;

        try {
          for (var _iterator4 = _getIterator(cmds), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
            var cmd = _step4.value;
            formatNode(ctx_1, function (ctx_2) {
              return function (cmd_1) {
                formatCommand(ctx_2, cmd_1);
              };
            }, cmd);
          }
        } catch (err) {
          _didIteratorError4 = true;
          _iteratorError4 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion4 && _iterator4.return) {
              _iterator4.return();
            }
          } finally {
            if (_didIteratorError4) {
              throw _iteratorError4;
            }
          }
        }
      };
    };

    return function (node_1) {
      formatNode(ctx, f, node_1);
    };
  })()(prog.Body);
  return join("", ctx.Strings);
}
export function formatWhiteAfterExpr(nd) {
  var wa = nd.Node.Case === "Variable" ? append(nd.Node.Fields[0].WhiteAfter, nd.WhiteAfter) : nd.WhiteAfter;
  return join("", toList(delay(function () {
    return map(function (t) {
      return formatToken(t.Token);
    }, wa);
  })));
}
export function formatWhiteBeforeExpr(nd) {
  var wa = void 0;
  var $var305 = nd.Node.Case === "Variable" ? [0, nd.Node.Fields[0]] : nd.Node.Case === "Member" ? nd.Node.Fields[1].Node.Case === "Variable" ? [1, nd.Node.Fields[1], nd.Node.Fields[1].Node.Fields[0]] : [2] : [2];

  switch ($var305[0]) {
    case 0:
      wa = append(nd.WhiteBefore, $var305[1].WhiteBefore);
      break;

    case 1:
      wa = append(nd.WhiteBefore, append($var305[1].WhiteBefore, $var305[2].WhiteBefore));
      break;

    case 2:
      wa = nd.WhiteBefore;
      break;
  }

  return join("", toList(delay(function () {
    return map(function (t) {
      return formatToken(t.Token);
    }, wa);
  })));
}
export function formatEntityKind(_arg1) {
  if (_arg1.Case === "Variable") {
    return "variable";
  } else if (_arg1.Case === "Binding") {
    return "binding";
  } else if (_arg1.Case === "Operator") {
    return formatToken(new TokenKind("Operator", [_arg1.Fields[1]])) + " operator";
  } else if (_arg1.Case === "List") {
    return "list";
  } else if (_arg1.Case === "Constant") {
    if (_arg1.Fields[0].Case === "Number") {
      return fsFormat("number `%f`")(function (x) {
        return x;
      })(_arg1.Fields[0].Fields[0]);
    } else if (_arg1.Fields[0].Case === "String") {
      return fsFormat("string `%s`")(function (x) {
        return x;
      })(_arg1.Fields[0].Fields[0]);
    } else if (_arg1.Fields[0].Case === "Boolean") {
      if (_arg1.Fields[0].Fields[0]) {
        return "`true` value";
      } else {
        return "`false` value";
      }
    } else {
      return "empty value";
    }
  } else if (_arg1.Case === "Function") {
    return "function";
  } else if (_arg1.Case === "LetCommand") {
    return "let command";
  } else if (_arg1.Case === "RunCommand") {
    return "run command";
  } else if (_arg1.Case === "Program") {
    return "program";
  } else if (_arg1.Case === "Root") {
    return "root";
  } else if (_arg1.Case === "CallSite") {
    return "call site";
  } else if (_arg1.Case === "NamedParam") {
    return "named param";
  } else if (_arg1.Case === "Call") {
    return "call";
  } else if (_arg1.Case === "ArgumentList") {
    return "argument list";
  } else if (_arg1.Case === "Member") {
    return "member";
  } else if (_arg1.Case === "MemberAccess") {
    return "member access";
  } else if (_arg1.Case === "MemberName") {
    return "member name";
  } else if (_arg1.Case === "Placeholder") {
    return "placeholder";
  } else {
    return "global value";
  }
}
export function entityCodeNameAndAntecedents(_arg1) {
  if (_arg1.Case === "Program") {
    return [1, _arg1.Fields[0], "<program>"];
  } else if (_arg1.Case === "RunCommand") {
    return [2, ofArray([_arg1.Fields[0]]), "<do>"];
  } else if (_arg1.Case === "LetCommand") {
    return [3, ofArray([_arg1.Fields[0], _arg1.Fields[1]]), "<let>"];
  } else if (_arg1.Case === "Operator") {
    return [4, ofArray([_arg1.Fields[0], _arg1.Fields[2]]), formatToken(new TokenKind("Operator", [_arg1.Fields[1]]))];
  } else if (_arg1.Case === "List") {
    return [5, _arg1.Fields[0], "<list>"];
  } else if (_arg1.Case === "Constant") {
    if (_arg1.Fields[0].Case === "Number") {
      return [7, new List(), String(_arg1.Fields[0].Fields[0])];
    } else if (_arg1.Fields[0].Case === "Boolean") {
      return [8, new List(), String(_arg1.Fields[0].Fields[0])];
    } else if (_arg1.Fields[0].Case === "Empty") {
      return [9, new List(), "<empty>"];
    } else {
      return [6, new List(), _arg1.Fields[0].Fields[0]];
    }
  } else if (_arg1.Case === "Function") {
    return [10, ofArray([_arg1.Fields[0], _arg1.Fields[1]]), "<function>"];
  } else if (_arg1.Case === "GlobalValue") {
    return [11, new List(), _arg1.Fields[0].Name];
  } else if (_arg1.Case === "Variable") {
    return [12, ofArray([_arg1.Fields[1]]), _arg1.Fields[0].Name];
  } else if (_arg1.Case === "Binding") {
    return [13, ofArray([_arg1.Fields[1]]), _arg1.Fields[0].Name];
  } else if (_arg1.Case === "ArgumentList") {
    return [14, _arg1.Fields[0], "<args>"];
  } else if (_arg1.Case === "Call") {
    return [15, ofArray([_arg1.Fields[0], _arg1.Fields[1]]), "<call>"];
  } else if (_arg1.Case === "Member") {
    return [16, ofArray([_arg1.Fields[0], _arg1.Fields[1]]), "<member>"];
  } else if (_arg1.Case === "MemberAccess") {
    return [17, ofArray([_arg1.Fields[0]]), "<member access>"];
  } else if (_arg1.Case === "NamedParam") {
    return [18, ofArray([_arg1.Fields[1]]), _arg1.Fields[0].Name];
  } else if (_arg1.Case === "Placeholder") {
    return [19, ofArray([_arg1.Fields[1]]), _arg1.Fields[0].Name];
  } else if (_arg1.Case === "CallSite") {
    if (_arg1.Fields[1].Case === "Choice2Of2") {
      return [21, ofArray([_arg1.Fields[0]]), String(_arg1.Fields[1].Fields[0])];
    } else {
      return [20, ofArray([_arg1.Fields[0]]), _arg1.Fields[1].Fields[0]];
    }
  } else if (_arg1.Case === "MemberName") {
    return [22, new List(), _arg1.Fields[0].Name];
  } else {
    return [0, new List(), "<root>"];
  }
}
export function lastChainElement(ent) {
  lastChainElement: while (true) {
    if (ent.Kind.Case === "Variable") {
      return ent;
    } else if (ent.Kind.Case === "Member") {
      return ent.Kind.Fields[1];
    } else if (ent.Kind.Case === "MemberAccess") {
      ent = ent.Kind.Fields[0];
      continue lastChainElement;
    } else {
      return ent;
    }
  }
}

function Entity_get_Antecedents() {
  var patternInput = entityCodeNameAndAntecedents(this.Kind);
  return patternInput[1];
}

export { Entity_get_Antecedents as Entity$2E$get_Antecedents };

function Entity_get_Name() {
  var patternInput = entityCodeNameAndAntecedents(this.Kind);
  return patternInput[2];
}

export { Entity_get_Name as Entity$2E$get_Name };
export function formatType(_arg1) {
  if (_arg1.Case === "Primitive") {
    if (_arg1.Fields[0].Case === "Date") {
      return "date";
    } else if (_arg1.Fields[0].Case === "Number") {
      return "number";
    } else if (_arg1.Fields[0].Case === "String") {
      return "string";
    } else if (_arg1.Fields[0].Case === "Unit") {
      return "unit";
    } else {
      return "bool";
    }
  } else if (_arg1.Case === "Object") {
    try {
      var mem = _arg1.Fields[0].Members;
      var mems = join(", ", map(function (m) {
        return m.Name;
      }, truncate(5, mem)));
      return "{ " + (mem.length > 5 ? mems + ", ..." : mems + " }");
    } catch (matchValue) {
      return "{ members }";
    }
  } else if (_arg1.Case === "Method") {
    var tout = void 0;

    var matchValue_1 = _arg1.Fields[1](toList(delay(function () {
      return map(function (ma) {
        return [ma.Type, null];
      }, _arg1.Fields[0]);
    })));

    if (matchValue_1 != null) {
      tout = formatType(matchValue_1);
    } else {
      tout = "?";
    }

    var tin = join(", ", toList(delay(function () {
      return map(function (ma_1) {
        return fsFormat("%s%s%s:%s")(function (x) {
          return x;
        })(ma_1.Optional ? "?" : "")(ma_1.Static ? "!" : "")(ma_1.Name)(formatType(ma_1.Type));
      }, _arg1.Fields[0]);
    })));
    return "(" + tin + ") -> " + tout;
  } else if (_arg1.Case === "List") {
    return "list<" + formatType(_arg1.Fields[0]) + ">";
  } else if (_arg1.Case === "Any") {
    return "any";
  } else {
    return "delayed type";
  }
}
export function formatTypeInfo(_arg1) {
  if (_arg1.Case === "Primitive") {
    if (_arg1.Fields[0].Case === "Date") {
      return "date";
    } else if (_arg1.Fields[0].Case === "Number") {
      return "number";
    } else if (_arg1.Fields[0].Case === "String") {
      return "string";
    } else if (_arg1.Fields[0].Case === "Unit") {
      return "unit";
    } else {
      return "bool";
    }
  } else if (_arg1.Case === "Object") {
    return "object type";
  } else if (_arg1.Case === "Method") {
    return "method type";
  } else if (_arg1.Case === "List") {
    return "list type";
  } else if (_arg1.Case === "Any") {
    return "unknown";
  } else {
    return "delayed type";
  }
}
export function rebuildExprNode(e, es, ns) {
  var matchValue = [e, es, ns];
  var $var306 = matchValue[0].Case === "List" ? matchValue[2].tail == null ? [1, matchValue[1]] : [11] : matchValue[0].Case === "Function" ? matchValue[1].tail != null ? matchValue[1].tail.tail == null ? matchValue[2].tail != null ? matchValue[2].tail.tail == null ? [2, matchValue[1].head, matchValue[2].head] : [12] : [12] : [12] : [12] : matchValue[0].Case === "Member" ? matchValue[1].tail != null ? matchValue[1].tail.tail != null ? matchValue[1].tail.tail.tail == null ? matchValue[2].tail == null ? [3, matchValue[1].head, matchValue[1].tail.head] : [9] : [9] : [9] : [9] : matchValue[0].Case === "Binary" ? matchValue[1].tail != null ? matchValue[1].tail.tail != null ? matchValue[1].tail.tail.tail == null ? matchValue[2].tail == null ? [4, matchValue[1].head, matchValue[1].tail.head, matchValue[0].Fields[1]] : [13] : [13] : [13] : [13] : matchValue[0].Case === "Call" ? matchValue[1].tail != null ? [5, matchValue[0].Fields[1], matchValue[1].head, matchValue[1].tail, matchValue[2]] : [10] : matchValue[0].Case === "Variable" ? matchValue[1].tail == null ? matchValue[2].tail != null ? matchValue[2].tail.tail == null ? [6, matchValue[2].head] : [8] : [8] : [8] : matchValue[0].Case === "Number" ? [14] : matchValue[0].Case === "Boolean" ? [14] : matchValue[0].Case === "String" ? [14] : matchValue[0].Case === "Empty" ? [14] : matchValue[1].tail != null ? matchValue[1].tail.tail == null ? matchValue[2].tail != null ? matchValue[2].tail.tail == null ? [0, matchValue[1].head, matchValue[2].head] : [7] : [7] : [7] : [7];

  switch ($var306[0]) {
    case 0:
      return new Expr("Placeholder", [$var306[2], $var306[1]]);

    case 1:
      return new Expr("List", [$var306[1]]);

    case 2:
      return new Expr("Function", [$var306[2], $var306[1]]);

    case 3:
      return new Expr("Member", [$var306[1], $var306[2]]);

    case 4:
      return new Expr("Binary", [$var306[1], $var306[3], $var306[2]]);

    case 5:
      var rebuildArgs = function rebuildArgs(args) {
        return function (es_1) {
          return function (ns_1) {
            var matchValue_1 = [args, es_1, ns_1];
            var $var307 = matchValue_1[0].tail == null ? matchValue_1[1].tail == null ? matchValue_1[2].tail == null ? [2] : [3] : [3] : matchValue_1[0].head.Name != null ? matchValue_1[1].tail != null ? matchValue_1[2].tail != null ? [1, matchValue_1[0].tail, matchValue_1[1].head, matchValue_1[1].tail, matchValue_1[2].head, matchValue_1[2].tail] : [3] : [3] : matchValue_1[1].tail != null ? [0, matchValue_1[0].tail, matchValue_1[1].head, matchValue_1[1].tail, matchValue_1[2]] : [3];

            switch ($var307[0]) {
              case 0:
                return new List(new Argument(null, $var307[2]), rebuildArgs($var307[1])($var307[3])($var307[4]));

              case 1:
                return new List(new Argument($var307[4], $var307[2]), rebuildArgs($var307[1])($var307[3])($var307[5]));

              case 2:
                return new List();

              case 3:
                throw new Error("rebuildExprNode: Wrong call length");
            }
          };
        };
      };

      return new Expr("Call", [$var306[2], function () {
        var _Node = rebuildArgs($var306[1].Node)($var306[3])($var306[4]);

        return new _Node_1($var306[1].WhiteBefore, $var306[1].WhiteAfter, $var306[1].Range, _Node, $var306[1].Entity);
      }()]);

    case 6:
      return new Expr("Variable", [$var306[1]]);

    case 7:
      throw new Error("rebuildExprNode: Wrong placeholder length");

    case 8:
      throw new Error("rebuildExprNode: Wrong variable length");

    case 9:
      throw new Error("rebuildExprNode: Wrong member length");

    case 10:
      throw new Error("rebuildExprNode: Wrong call length");

    case 11:
      throw new Error("rebuildExprNode: Wrong list length");

    case 12:
      throw new Error("rebuildExprNode: Wrong function length");

    case 13:
      throw new Error("rebuildExprNode: Wrong binary operator argument length");

    case 14:
      throw new Error("rebuildExprNode: Not a node");
  }
}

function _ExprLeaf_ExprNode_(e) {
  var $var308 = e.Case === "Member" ? [1] : e.Case === "Call" ? [2] : e.Case === "Variable" ? [3] : e.Case === "List" ? [4] : e.Case === "Function" ? [5] : e.Case === "Binary" ? [6] : e.Case === "Number" ? [7] : e.Case === "Boolean" ? [7] : e.Case === "String" ? [7] : e.Case === "Empty" ? [7] : [0];

  switch ($var308[0]) {
    case 0:
      return new Choice("Choice2Of2", [[ofArray([e.Fields[1]]), ofArray([e.Fields[0]])]]);

    case 1:
      return new Choice("Choice2Of2", [[ofArray([e.Fields[0], e.Fields[1]]), new List()]]);

    case 2:
      return new Choice("Choice2Of2", [[new List(e.Fields[0], toList(delay(function () {
        return map(function (a) {
          return a.Value;
        }, e.Fields[1].Node);
      }))), choose(function (a_1) {
        return a_1.Name;
      }, e.Fields[1].Node)]]);

    case 3:
      return new Choice("Choice2Of2", [[new List(), ofArray([e.Fields[0]])]]);

    case 4:
      return new Choice("Choice2Of2", [[e.Fields[0], new List()]]);

    case 5:
      return new Choice("Choice2Of2", [[ofArray([e.Fields[1]]), ofArray([e.Fields[0]])]]);

    case 6:
      return new Choice("Choice2Of2", [[ofArray([e.Fields[0], e.Fields[2]]), new List()]]);

    case 7:
      return new Choice("Choice1Of2", [null]);
  }
}

export { _ExprLeaf_ExprNode_ as $7C$ExprLeaf$7C$ExprNode$7C$ };

function _FindMember___(name, obj) {
  return tryPick(function (m) {
    return m.Name === name.Name ? m : null;
  }, obj.Members);
}

export { _FindMember___ as $7C$FindMember$7C$_$7C$ };
export function pickMetaByType(ctx, typ, metas) {
  return tryPick(function (m) {
    return (m.Context === ctx ? m.Type === typ : false) ? m.Data : null;
  }, metas);
}
//# sourceMappingURL=astops.js.map