import _Array$from from "babel-runtime/core-js/array/from";
import { TokenKind, Error as _Error } from "./ast";
import { join, fsFormat } from "fable-core/String";
import { formatToken, formatTypeInfo, formatTokenInfo } from "./astops";
import { map } from "fable-core/List";
import { map as map_1, delay, toList } from "fable-core/Seq";
export var Tokenizer = function (__exports) {
  var inputEndInsideString = __exports.inputEndInsideString = function (rng, s) {
    var _Number = 101;
    return new _Error(_Number, fsFormat("Missing \" at the end of the input. The string \"%s\" ends without closing double-quote.")(function (x) {
      return x;
    })(s), rng);
  };

  var missingClosingQuote = __exports.missingClosingQuote = function (rng, q) {
    var _Number = 102;
    return new _Error(_Number, fsFormat("Quoted identifier '%s' is missing closing quote.")(function (x) {
      return x;
    })(q), rng);
  };

  var unexpectedCharacter = __exports.unexpectedCharacter = function (rng, c) {
    var _Number = 103;
    return new _Error(_Number, fsFormat("Unexcpected character '%s' in the input.")(function (x) {
      return x;
    })(c), rng);
  };

  return __exports;
}({});
export var Parser = function (__exports) {
  var unexpectedTokenAfterDot = __exports.unexpectedTokenAfterDot = function (rng, tok) {
    var _Number = 201;
    return new _Error(_Number, fsFormat("Unexpected %s after '.' in method chain")(function (x) {
      return x;
    })(formatTokenInfo(tok)), rng);
  };

  var unexpectedScopeEndAfterDot = __exports.unexpectedScopeEndAfterDot = function (rng, tok) {
    var _Number = 202;
    return new _Error(_Number, fsFormat("Unexpected end of scope after %s. Did you forget to indent the rest of the member chain?")(function (x) {
      return x;
    })(formatTokenInfo(tok)), rng);
  };

  var unexpectedTokenInPlaceholder = __exports.unexpectedTokenInPlaceholder = function (rng, tok) {
    var _Number = 203;
    return new _Error(_Number, fsFormat("Unexpected token '%s' in placeholder expression. Shold be `[ident: <expr>]`")(function (x) {
      return x;
    })(formatTokenInfo(tok)), rng);
  };

  var unexpectedScopeEndInPlaceholder = __exports.unexpectedScopeEndInPlaceholder = function (rng, tok) {
    if (tok != null) {
      var _Number = 204;
      return new _Error(_Number, fsFormat("Unexpected end of scope in placeholder after %s. Did you forget to indent the body of the plceholder?")(function (x) {
        return x;
      })(formatTokenInfo(tok)), rng);
    } else {
      var _Number_1 = 204;
      return new _Error(_Number_1, "Unexpected end of scope in placeholder after expression. Did you forget to indent the body of the plceholder?", rng);
    }
  };

  var unexpectedEndOfPlaceholder = __exports.unexpectedEndOfPlaceholder = function (rng) {
    var _Number = 205;
    return new _Error(_Number, "Incomplete placeholder expression. Shold be `[ident: <expr>]`", rng);
  };

  var unexpectedEndAfterOperator = __exports.unexpectedEndAfterOperator = function (rng, op) {
    var _Number = 206;
    return new _Error(_Number, fsFormat("Unexpected token after operator '%s'. Expected an expression or closing parenthesis.")(function (x) {
      return x;
    })(formatTokenInfo(op)), rng);
  };

  var unexpectedTokenInArgList = __exports.unexpectedTokenInArgList = function (rng, tok) {
    var _Number = 207;
    return new _Error(_Number, fsFormat("Unexpected token '%s' in list of call arguments")(function (x) {
      return x;
    })(formatTokenInfo(tok)), rng);
  };

  var unexpectedScopeEndInArgList = __exports.unexpectedScopeEndInArgList = function (rng) {
    var _Number = 208;
    return new _Error(_Number, "Unexpected end of argument list. Did you forget to indent the arguments?", rng);
  };

  var unexpectedTokenInParenthesizedExpr = __exports.unexpectedTokenInParenthesizedExpr = function (rng, tok) {
    var _Number = 209;
    return new _Error(_Number, fsFormat("Unexpected token '%s' in parenthesized expression. Are you missing ')'?")(function (x) {
      return x;
    })(formatTokenInfo(tok)), rng);
  };

  var unexpectedScopeEndInParenthesizedExpr = __exports.unexpectedScopeEndInParenthesizedExpr = function (rng) {
    var _Number = 210;
    return new _Error(_Number, "Unexpected end of nested expression in `(`. Did you forget to indent the body of the expression?", rng);
  };

  var missingParenthesizedExpr = __exports.missingParenthesizedExpr = function (rng) {
    var _Number = 211;
    return new _Error(_Number, "The parenthesized expression (...) is missing body!", rng);
  };

  var unexpectedTokenInList = __exports.unexpectedTokenInList = function (rng, tok) {
    var _Number = 212;
    return new _Error(_Number, fsFormat("Unexpected token '%s' in list expression")(function (x) {
      return x;
    })(formatTokenInfo(tok)), rng);
  };

  var unexpectedScopeEndInList = __exports.unexpectedScopeEndInList = function (rng) {
    var _Number = 213;
    return new _Error(_Number, "Unexpected end of list expression. Did youu forget to indent the elements of the list?", rng);
  };

  var unexpectedTokenInLetBinding = __exports.unexpectedTokenInLetBinding = function (rng, tok) {
    var _Number = 214;
    return new _Error(_Number, fsFormat("Unexpected token '%s' in let declaration (should be `let name = expr`)")(function (x) {
      return x;
    })(formatTokenInfo(tok)), rng);
  };

  var missingBodyInLetBinding = __exports.missingBodyInLetBinding = function (rng) {
    var _Number = 215;
    return new _Error(_Number, "This let binding is missing body after equals (should be `let name = expr`", rng);
  };

  var unexpectedNestedTokenInCommand = __exports.unexpectedNestedTokenInCommand = function (rng, tok) {
    var _Number = 216;
    return new _Error(_Number, fsFormat("Unexpected indented token '%s' in command list. Try removing the indentation before the token.")(function (x) {
      return x;
    })(formatTokenInfo(tok)), rng);
  };

  var unexpectedTokenAfterFun = __exports.unexpectedTokenAfterFun = function (rng, tok) {
    var _Number = 217;
    return new _Error(_Number, fsFormat("Unexpected token '%s' after `fun`. Expected variable name.")(function (x) {
      return x;
    })(formatTokenInfo(tok)), rng);
  };

  var missingArrowInFunc = __exports.missingArrowInFunc = function (rng) {
    var _Number = 218;
    return new _Error(_Number, "Missing arrow after variable in function definition", rng);
  };

  var unexpectedScopeEndInFunc = __exports.unexpectedScopeEndInFunc = function (rng) {
    var _Number = 219;
    return new _Error(_Number, "Unexpected end of function declaration. Did you forget to indent the body of the function?", rng);
  };

  var missingBodyOfFunc = __exports.missingBodyOfFunc = function (rng) {
    var _Number = 220;
    return new _Error(_Number, "The function is missing body. Did you forget to indent the body of the function?", rng);
  };

  var unexpectedScopeEndInLet = __exports.unexpectedScopeEndInLet = function (rng) {
    var _Number = 221;
    return new _Error(_Number, "Unexpected end of let declaration. Did you forget to indent the body of the let declaration?", rng);
  };

  var exceptionWhileParsing = __exports.exceptionWhileParsing = function (rng, msg) {
    var _Number = 299;
    return new _Error(_Number, "Unexpected exception while parsing: " + msg, rng);
  };

  return __exports;
}({});
export var TypeChecker = function (__exports) {
  var numericOperatorExpectsNumbers = __exports.numericOperatorExpectsNumbers = function (op, idx, typs, typ, rng) {
    var _Number = 301;
    return new _Error(_Number, fsFormat("Both operands of binary operator '%s' should be %s but the %s operand was %s instead.")(function (x) {
      return x;
    })(join(" or ", map(function (_arg1) {
      return formatTypeInfo(_arg1);
    }, typs)))(formatToken(new TokenKind("Operator", [op])))(idx === 0 ? "left" : "right")(formatTypeInfo(typ)), rng);
  };

  var variableNotInScope = __exports.variableNotInScope = function (name, rng) {
    var _Number = 302;
    return new _Error(_Number, fsFormat("Variable '%s' is not in scope.")(function (x) {
      return x;
    })(name), rng);
  };

  var formatMembers = function formatMembers(members) {
    var members_1 = _Array$from(members);

    var suffix = members_1.length > 10 ? fsFormat(", (%d members)")(function (x) {
      return x;
    })(members_1.length - 10) : "";
    return join(", ", toList(delay(function () {
      return map_1(function (m) {
        return m.Name;
      }, members_1);
    }))) + suffix;
  };

  var memberMissing = __exports.memberMissing = function (name, members, rng) {
    var _Number = 303;
    return new _Error(_Number, fsFormat("Could not find property '%s' in the list '%s'.")(function (x) {
      return x;
    })(name)(formatMembers(members)), rng);
  };

  var notAnObject = __exports.notAnObject = function (name, typ, rng) {
    var _Number = 305;
    return new _Error(_Number, fsFormat("Type is not an object but %s and it does not have member `%s`")(function (x) {
      return x;
    })(formatTypeInfo(typ))(name), rng);
  };

  var listElementTypeDoesNotMatch = __exports.listElementTypeDoesNotMatch = function (listty, elty, rng) {
    var _Number = 306;
    return new _Error(_Number, fsFormat("The type of this list element is %s but it should be %s")(function (x) {
      return x;
    })(formatTypeInfo(elty))(formatTypeInfo(listty)), rng);
  };

  var nameBasedParamMustBeLast = __exports.nameBasedParamMustBeLast = function (rng) {
    var _Number = 307;
    return new _Error(_Number, "All named parameters must be at the end of parameter list.", rng);
  };

  var parameterMissingValue = __exports.parameterMissingValue = function (par, rng) {
    var _Number = 308;
    return new _Error(_Number, fsFormat("Required parameter `%s` is not given a value.")(function (x) {
      return x;
    })(par), rng);
  };

  var notAnMethod = __exports.notAnMethod = function (name, typ, rng) {
    var _Number = 309;
    return new _Error(_Number, fsFormat("The type of member %s is `%s` and not a method and so it cannot be called.")(function (x) {
      return x;
    })(name)(formatTypeInfo(typ)), rng);
  };

  var parameterConflict = __exports.parameterConflict = function (rng) {
    var _Number = 310;
    return new _Error(_Number, "Invalid argument type", rng);
  };

  var callMissingInstance = __exports.callMissingInstance = function (name, rng) {
    var _Number = 311;
    return new _Error(_Number, fsFormat("The `%s` property access or call is missing an instance")(function (x) {
      return x;
    })(name), rng);
  };

  return __exports;
}({});
//# sourceMappingURL=errors.js.map