import _Map from "babel-runtime/core-js/map";
import _Array$from from "babel-runtime/core-js/array/from";
import _defineProperty from "babel-runtime/helpers/defineProperty";
import _Symbol from "fable-core/Symbol";
import { tokenize as _tokenize } from "./parser/tokenizer";
import { fold, toList, tryPick, range, map, delay } from "fable-core/Seq";
import { startAsPromise } from "fable-core/Async";
import { Log } from "../libraries/common";
import { LocationMapper } from "./common/locations";
import { escapeIdent, pickMetaByType } from "./ast/astops";
import { reverse } from "fable-core/List";
import List from "fable-core/List";
import { join } from "fable-core/String";
import { singleton } from "fable-core/AsyncBuilder";
import { editor, languages } from "monaco";
var i = 1;
eval('monaco$' + i + ' = monaco');
monaco = monaco;
export var noState = _defineProperty({
  clone: function () {
    return this;
  },
  equals: function (other) {
    return true;
  }
}, _Symbol.reflection, function () {
  return {
    interfaces: ["Fable.Import.monaco.languages.IState"]
  };
});
export function getColorClass(_arg1) {
  var $var557 = _arg1.Case === "String" ? [0] : _arg1.Case === "QIdent" ? [1] : _arg1.Case === "Ident" ? [1] : _arg1.Case === "Dot" ? [2] : _arg1.Case === "Let" ? [3] : _arg1.Case === "Boolean" ? [3] : _arg1.Case === "Fun" ? [3] : _arg1.Case === "Arrow" ? [3] : _arg1.Case === "Number" ? [4] : [5];

  switch ($var557[0]) {
    case 0:
      return "string";

    case 1:
      return "ident";

    case 2:
      return "operator";

    case 3:
      return "keyword";

    case 4:
      return "number";

    case 5:
      return "";
  }
}
export var tokensProvider = _defineProperty({
  tokenize: function (line, state) {
    var tokens = {};
    tokens.endState = noState;
    tokens.tokens = [];

    var patternInput = _tokenize(line);

    for (var idx = 0; idx <= patternInput[0].length - 1; idx++) {
      var t = patternInput[0][idx];
      var tok = {};
      tok.startIndex = t.Range.Start;
      tok.scopes = getColorClass(t.Token);
      tokens.tokens.push(tok);
    }

    return tokens;
  },
  getInitialState: function () {
    return noState;
  }
}, _Symbol.reflection, function () {
  return {
    interfaces: ["Fable.Import.monaco.languages.TokensProvider"]
  };
});
export function createCompletionProvider(getService) {
  return _defineProperty({
    get triggerCharacters() {
      return _Array$from(_Array$from(delay(function () {
        return map(function (i) {
          return String.fromCharCode(i);
        }, range(0, 255));
      })));
    },

    provideCompletionItems: function (model, position, token) {
      return function (arg00) {
        return startAsPromise(arg00);
      }(function (builder_) {
        return builder_.Delay(function () {
          return builder_.TryWith(builder_.Delay(function () {
            var svc = getService(model.uri.toString());
            var input = model.getValue(1, false);
            Log.event("editor", "completions", "", {
              source: input,
              position: position
            });
            var conv = new LocationMapper(input);
            var loc = conv.LineColToAbsolute(~~position.lineNumber, ~~position.column);
            return builder_.Bind(svc.TypeCheck(input), function (_arg1) {
              var optMembers = tryPick(function (tupledArg) {
                var $var558 = tupledArg[1].Kind.Case === "Member" ? tupledArg[1].Kind.Fields[0].Type != null ? tupledArg[1].Kind.Fields[1].Kind.Case === "MemberName" ? function () {
                  var t = tupledArg[1].Kind.Fields[0].Type;
                  var n = tupledArg[1].Kind.Fields[1].Kind.Fields[0];

                  if (loc >= tupledArg[0].Start) {
                    return loc <= tupledArg[0].End + 1;
                  } else {
                    return false;
                  }
                }() ? [0, tupledArg[1].Kind.Fields[1].Kind.Fields[0], tupledArg[1].Kind.Fields[0].Type] : [1] : [1] : [1] : [1];

                switch ($var558[0]) {
                  case 0:
                    Log.trace("completions", "Antecedant at current location (member '%s'): %O", $var558[1].Name, $var558[2]);

                    if ($var558[2].Case === "Object") {
                      return [$var558[1].Name, tupledArg[0], $var558[2].Fields[0].Members];
                    } else {
                      return null;
                    }

                  case 1:
                    var $var559 = tupledArg[1].Kind.Case === "Member" ? tupledArg[1].Kind.Fields[0].Type != null ? tupledArg[1].Kind.Fields[1].Kind.Case === "MemberName" ? [0, tupledArg[1].Kind.Fields[1].Kind.Fields[0], tupledArg[1].Kind.Fields[0].Type] : [1] : [1] : [1];

                    switch ($var559[0]) {
                      case 0:
                        Log.trace("completions", "Ignoring '%s' at location %s-%s (current=%s)", $var559[1].Name, tupledArg[0].Start, tupledArg[0].End, loc);
                        return null;

                      case 1:
                        return null;
                    }

                }
              }, _arg1[1].Entities);

              var convertRange = function convertRange(rng) {
                var patternInput = conv.AbsoluteToLineCol(rng.Start);
                var patternInput_1 = conv.AbsoluteToLineCol(rng.End);
                var res = {};
                res.startColumn = patternInput[1];
                res.startLineNumber = patternInput[0];
                res.endColumn = patternInput_1[1] + 1;
                res.endLineNumber = patternInput_1[0];
                return res;
              };

              if (optMembers != null) {
                var nameRange = optMembers[1];
                var members = optMembers[2];
                var currentName = optMembers[0];
                var nameRange_1 = convertRange(nameRange);
                Log.trace("completions", "providing %s members at %O", members.length, nameRange_1);
                var members_1 = members.filter(function (m) {
                  var matchValue = pickMetaByType("http://schema.thegamma.net", "CompletionItem", m.Metadata);

                  if (matchValue != null) {
                    return !matchValue["hidden"];
                  } else {
                    return true;
                  }
                });
                var completion = toList(delay(function () {
                  return map(function (m_1) {
                    var ci = {};
                    var k = m_1.Type.Case === "Method" ? 1 : 9;
                    ci.kind = k;
                    ci.label = m_1.Name;
                    ci.insertText = escapeIdent(m_1.Name);
                    ci.filterText = escapeIdent(currentName) + m_1.Name;

                    if (m_1.Type.Case === "Method") {
                      var patternInput_2 = function () {
                        var folder = function folder(tupledArg_1) {
                          return function (s) {
                            if (tupledArg_1[1].length > 100) {
                              return [new List(tupledArg_1[1], tupledArg_1[0]), s];
                            } else {
                              return [tupledArg_1[0], tupledArg_1[1] === "" ? s : tupledArg_1[1] + "," + s];
                            }
                          };
                        };

                        var arg = [new List(), ""];
                        return function (source) {
                          return fold(function ($var560, $var561) {
                            return folder($var560)($var561);
                          }, [arg[0], arg[1]], source);
                        };
                      }()(toList(delay(function () {
                        return map(function (ma) {
                          return (ma.Optional ? "?" : "") + ma.Name;
                        }, m_1.Type.Fields[0]);
                      })));

                      var args = join(",\n", reverse(new List(patternInput_2[1], patternInput_2[0])));
                      ci.documentation = "(" + args + ")";
                    }

                    var eo = {};
                    eo.text = escapeIdent(m_1.Name);
                    eo.range = nameRange_1;
                    ci.textEdit = eo;
                    return ci;
                  }, members_1);
                }));
                Log.trace("completions", "returning %O", _Array$from(completion));
                return builder_.Return(_Array$from(completion));
              } else {
                Log.trace("completions", "no members at %s", loc);
                return builder_.Return(_Array$from(new List()));
              }
            });
          }), function (_arg2) {
            Log.exn("completions", "completions failed %O", _arg2);
            return builder_.Return([]);
          });
        });
      }(singleton));
    },
    resolveCompletionItem: function (item, token) {
      return item;
    }
  }, _Symbol.reflection, function () {
    return {
      interfaces: ["Fable.Import.monaco.languages.CompletionItemProvider"]
    };
  });
}
export var createdEditors = new _Map();
export function getService(uri) {
  return createdEditors.get(uri);
}
export function setupMonacoServices() {
  var lang = {};
  lang.id = "thegamma";
  languages.register(lang);
  languages.setTokensProvider("thegamma", tokensProvider);
  languages.registerCompletionItemProvider("thegamma", createCompletionProvider(function (uri) {
    return getService(uri);
  }));
}
export function createMonacoEditor(id, code, svc, customize) {
  if (createdEditors.size === 0) {
    setupMonacoServices();
  }

  var services = {};
  var options = {};
  var scroll = {};
  scroll.vertical = "none";
  scroll.horizontal = "auto";
  options.scrollbar = scroll;
  options.value = code;
  options.language = "thegamma";
  options.lineNumbersMinChars = 3;
  options.contextmenu = false;
  options.scrollBeyondLastLine = false;
  options.overviewRulerLanes = 0;
  customize(options);
  var ed = editor.create(document.getElementById(id), options, services);
  createdEditors.set(ed.getModel().uri.toString(), svc);
  return ed;
}
//# sourceMappingURL=monaco.js.map