import _defineProperty from "babel-runtime/helpers/defineProperty";
import _classCallCheck from "babel-runtime/helpers/classCallCheck";
import _createClass from "babel-runtime/helpers/createClass";
import { setType } from "fable-core/Symbol";
import _Symbol from "fable-core/Symbol";
import { equals, defaultArg, Interface, compareRecords, equalsRecords, equalsUnions, Any } from "fable-core/Util";
import _Event from "fable-core/Event";
import { Error as _Error, Range as _Range, Program as Program_1 } from "./ast/ast";
import { formatType, Entity$2E$get_Name as Entity_get_Name, node as node_1 } from "./ast/astops";
import { map } from "fable-core/List";
import List from "fable-core/List";
import { Log, Async$2E$AwaitFuture$2E$Static as Async_AwaitFuture_Static, Async$2E$StartAsFuture$2E$Static as Async_StartAsFuture_Static } from "../libraries/common";
import { bindProgram, BindingResult, createContext } from "./analyzer/binder";
import { singleton } from "fable-core/AsyncBuilder";
import { start } from "fable-core/MailboxProcessor";
import { tryPick, toList } from "fable-core/Seq";
import { split } from "fable-core/String";
import { parseProgram } from "./parser/parser";
import { collectTypeErrors, typeCheckProgram } from "./analyzer/typechecker";
import { evaluate } from "./analyzer/interpreter";
import { LocationMapper } from "./common/locations";
import { sleep, startImmediate } from "fable-core/Async";
import { renderVirtual } from "../libraries/html";
import { patch, diff } from "virtual-dom";
import { updateLiveState, LiveState, LiveEvent } from "./live/live";
import { add } from "fable-core/Observable";
export var CheckingMessage = function () {
  function CheckingMessage(caseName, fields) {
    _classCallCheck(this, CheckingMessage);

    this.Case = caseName;
    this.Fields = fields;
  }

  _createClass(CheckingMessage, [{
    key: _Symbol.reflection,
    value: function () {
      return {
        type: "TheGamma.Services.CheckingMessage",
        interfaces: ["FSharpUnion", "System.IEquatable"],
        cases: {
          IsWellTyped: ["string", Any],
          TypeCheck: ["string", Any]
        }
      };
    }
  }, {
    key: "Equals",
    value: function (other) {
      return equalsUnions(this, other);
    }
  }]);

  return CheckingMessage;
}();
setType("TheGamma.Services.CheckingMessage", CheckingMessage);
export var Position = function () {
  function Position(line, column) {
    _classCallCheck(this, Position);

    this.Line = line;
    this.Column = column;
  }

  _createClass(Position, [{
    key: _Symbol.reflection,
    value: function () {
      return {
        type: "TheGamma.Services.Position",
        interfaces: ["FSharpRecord", "System.IEquatable", "System.IComparable"],
        properties: {
          Line: "number",
          Column: "number"
        }
      };
    }
  }, {
    key: "Equals",
    value: function (other) {
      return equalsRecords(this, other);
    }
  }, {
    key: "CompareTo",
    value: function (other) {
      return compareRecords(this, other);
    }
  }]);

  return Position;
}();
setType("TheGamma.Services.Position", Position);
export var LineRange = function () {
  function LineRange(start, end) {
    _classCallCheck(this, LineRange);

    this.Start = start;
    this.End = end;
  }

  _createClass(LineRange, [{
    key: _Symbol.reflection,
    value: function () {
      return {
        type: "TheGamma.Services.LineRange",
        interfaces: ["FSharpRecord", "System.IEquatable", "System.IComparable"],
        properties: {
          Start: Position,
          End: Position
        }
      };
    }
  }, {
    key: "Equals",
    value: function (other) {
      return equalsRecords(this, other);
    }
  }, {
    key: "CompareTo",
    value: function (other) {
      return compareRecords(this, other);
    }
  }]);

  return LineRange;
}();
setType("TheGamma.Services.LineRange", LineRange);
export function offsetToLocation(lines, offs, lengths) {
  offsetToLocation: while (true) {
    var $var550 = lengths.tail != null ? offs <= lengths.head ? [0, lengths.head, lengths.tail] : [1] : [1];

    switch ($var550[0]) {
      case 0:
        return new Position(lines, offs);

      case 1:
        if (lengths.tail == null) {
          return new Position(lines, offs);
        } else {
          lines = lines + 1;
          offs = offs - lengths.head - 1;
          lengths = lengths.tail;
          continue offsetToLocation;
        }

    }
  }
}
export function rangeToLoc(lengths, rng) {
  return new LineRange(offsetToLocation(1, rng.Start, lengths), offsetToLocation(1, rng.Start, lengths));
}
export var CheckingService = function () {
  _createClass(CheckingService, [{
    key: _Symbol.reflection,
    value: function () {
      return {
        type: "TheGamma.Services.CheckingService",
        properties: {
          ErrorsReported: Interface("Microsoft.FSharp.Control.IEvent")
        }
      };
    }
  }]);

  function CheckingService(article, globals) {
    var _this = this;

    _classCallCheck(this, CheckingService);

    this.globals = globals;
    this.errorsReported = new _Event();
    var emptyProg = new Program_1(node_1(new _Range(0, 0), new List()));
    this.bindingContext = Async_StartAsFuture_Static(function (builder_) {
      return builder_.Delay(function () {
        return builder_.Bind(Async_AwaitFuture_Static(_this.globals), function (_arg1) {
          return builder_.Return(createContext(_arg1, article));
        });
      });
    }(singleton));
    this.agent = start(function (inbox) {
      var loop = function loop(lastCode) {
        return function (lastResult) {
          return function (builder__1) {
            return builder__1.Delay(function () {
              return builder__1.Bind(inbox.receive(), function (_arg7) {
                if (_arg7.Case === "TypeCheck") {
                  if (_arg7.Fields[0] === lastCode) {
                    Log.trace("service", "Returning previous result");

                    _arg7.Fields[1].reply(lastResult);

                    return builder__1.ReturnFrom(loop(lastCode)(lastResult));
                  } else if (_arg7.Case === "TypeCheck") {
                    Log.trace("service", "Type checking source code");
                    return builder__1.Bind(_this.typeCheck(_arg7.Fields[0]), function (_arg9) {
                      if (_arg9 == null) {
                        _arg7.Fields[1].reply([false, new BindingResult([]), emptyProg]);

                        return builder__1.ReturnFrom(loop(lastCode)(lastResult));
                      } else {
                        var prog = _arg9[0];
                        var errors = _arg9[2];
                        var ents = _arg9[1];

                        _this.errorsReported.Trigger([_arg7.Fields[0], errors]);

                        var result = [errors.length === 0, ents, prog];

                        _arg7.Fields[1].reply(result);

                        return builder__1.ReturnFrom(loop(_arg7.Fields[0])(result));
                      }
                    });
                  } else {
                    throw new Error("C:\\Tomas\\Public\\the-gamma\\thegamma-script\\src\\thegamma\\services.fs", 64, 12);
                  }
                } else {
                  return builder__1.Bind(_this.typeCheck(_arg7.Fields[0]), function (_arg8) {
                    return builder__1.Combine(function () {
                      var $var551 = _arg8 != null ? function () {
                        var errs = _arg8[2];
                        return errs.length === 0;
                      }() ? [0, _arg8[2]] : [1] : [1];

                      switch ($var551[0]) {
                        case 0:
                          _arg7.Fields[1].reply(true);

                          return builder__1.Zero();

                        case 1:
                          _arg7.Fields[1].reply(false);

                          return builder__1.Zero();
                      }
                    }(), builder__1.Delay(function () {
                      return builder__1.ReturnFrom(loop(lastCode)(lastResult));
                    }));
                  });
                }
              });
            });
          }(singleton);
        };
      };

      return loop("")([false, new BindingResult([]), emptyProg]);
    });
  }

  _createClass(CheckingService, [{
    key: "TypeCheck",
    value: function (code) {
      return this.agent.postAndAsyncReply(function (ch) {
        return new CheckingMessage("TypeCheck", [code, ch]);
      });
    }
  }, {
    key: "IsWellTyped",
    value: function (code) {
      return this.agent.postAndAsyncReply(function (ch) {
        return new CheckingMessage("IsWellTyped", [code, ch]);
      });
    }
  }, {
    key: "errorsToLineCol",
    value: function (code, errors) {
      var lengths = map(function (l) {
        return l.length;
      }, toList(split(code, "\n")));
      return errors.map(function (e) {
        return new _Error(e.Number, e.Message, rangeToLoc(lengths, e.Range));
      });
    }
  }, {
    key: "typeCheck",
    value: function (code) {
      var _this2 = this;

      return function (builder_) {
        return builder_.Delay(function () {
          return builder_.Bind(Async_AwaitFuture_Static(_this2.globals), function (_arg2) {
            Log.trace("service", "Evaluated globals");
            return builder_.Combine(builder_.For(_arg2, function (_arg3) {
              Log.trace("service", "Global value '%s' : %O = %O", Entity_get_Name.bind(_arg3)(), defaultArg(_arg3.Type, null, function (_arg1) {
                return formatType(_arg1);
              }), _arg3.Value);
              return builder_.Zero();
            }), builder_.Delay(function () {
              return builder_.TryWith(builder_.Delay(function () {
                var patternInput = parseProgram(code);
                return builder_.Bind(Async_AwaitFuture_Static(_this2.bindingContext), function (_arg4) {
                  var patternInput_1 = bindProgram(_arg4, patternInput[0]);
                  return builder_.Bind(typeCheckProgram(_arg2, patternInput_1[1], function (e) {
                    return evaluate(_arg2, e);
                  }, patternInput_1[0]), function () {
                    var typeErrors = collectTypeErrors(patternInput_1[0]);
                    Log.trace("service", "Type checking completed");
                    var errors_1 = void 0;
                    var errors = patternInput[1].concat(typeErrors);
                    errors_1 = _this2.errorsToLineCol(code, errors);
                    return builder_.Return([patternInput[0], patternInput_1[1], errors_1]);
                  });
                });
              }), function (_arg6) {
                Log.exn("service", "Type checking failed: %O", _arg6);
                return builder_.Return(null);
              });
            }));
          });
        });
      }(singleton);
    }
  }, {
    key: "ErrorsReported",
    get: function () {
      return this.errorsReported.Publish;
    }
  }]);

  return CheckingService;
}();
setType("TheGamma.Services.CheckingService", CheckingService);
export var PreviewService = function () {
  _createClass(PreviewService, [{
    key: _Symbol.reflection,
    value: function () {
      return {
        type: "TheGamma.Services.PreviewService",
        properties: {
          ZoneHeight: "number",
          ZoneSizeChanged: Interface("Microsoft.FSharp.Control.IEvent")
        }
      };
    }
  }]);

  function PreviewService(checker, globals, ed, livePreviews) {
    var _this3 = this;

    _classCallCheck(this, PreviewService);

    this.checker = checker;
    this.globals = globals;
    this.ed = ed;
    this.livePreviews = livePreviews;
    this.zoneSizeChanged = new _Event();
    this.currentZone = null;
    this.zoneHeight = 0;
    this.tree = {};
    this.container = document.createElement("div");
    this.lastCode = "";
    this.lastMapper = new LocationMapper("");
    this.changingEditor = false;

    var trigger = function (ed_1) {
      return _this3.createLivePreview(ed_1);
    }(this.ed);

    this.ed.onDidChangeCursorPosition(function (ce) {
      if (!_this3.changingEditor) {
        var code = _this3.ed.getModel().getValue(1, false);

        Log.trace("live", "Cursor position changed: code <> lastCode = %s", code !== _this3.lastCode);

        (function (arg00) {
          startImmediate(arg00);
        })(function (builder_) {
          return builder_.Delay(function () {
            return builder_.Bind(_this3.getUpdateEventAfterChange(), function (_arg6) {
              trigger(_arg6);
              return builder_.Zero();
            });
          });
        }(singleton));
      }
    });
  }

  _createClass(PreviewService, [{
    key: "removeZone",
    value: function () {
      var matchValue = this.currentZone;

      if (matchValue == null) {} else {
        var id = matchValue[0];
        this.ed.changeViewZones(function (accessor) {
          accessor.removeZone(id);
        });
      }

      this.currentZone = null;
      this.zoneSizeChanged.Trigger(null);
    }
  }, {
    key: "createAndAddZone",
    value: function (endLine) {
      var _this4 = this;

      var zoneId = -1;
      var zone = {};
      var node = document.createElement('div');
      node.style.width = "1000px";
      node.style.height = "1000px";
      this.container = document.createElement('div');
      this.tree = {};
      node.appendChild(this.container);
      this.ed.changeViewZones(function (accessor) {
        var matchValue = _this4.currentZone;

        if (matchValue != null) {
          var id = matchValue[0];
          accessor.removeZone(id);
        }

        zone.afterLineNumber = endLine;
        zone.heightInPx = 1;
        zone.domNode = node;
        _this4.zoneHeight = 1;
        zoneId = accessor.addZone(zone);
        _this4.currentZone = [zoneId, zone];
      });
    }
  }, {
    key: "updateZones",
    value: function (trigger, liveState) {
      var _this5 = this;

      var dom = defaultArg(liveState.CurrentPreview, null, function (p) {
        return p.Render(trigger)(liveState);
      });

      if (dom != null) {
        if (function () {
          return _this5.currentZone == null;
        }(null)) {
          var endLine = dom.Line;
          this.createAndAddZone(endLine);
        }

        var patternInput = this.currentZone;
        var newTree = renderVirtual(dom.Preview);
        var patches = diff(this.tree, newTree);
        this.container = patch(this.container, patches);
        this.tree = newTree;

        var waitForActualHeight = function waitForActualHeight(n) {
          return function (builder_) {
            return builder_.Delay(function () {
              var newHeight = _this5.container.clientHeight;

              if (n === 10 ? true : newHeight !== 0) {
                return builder_.Return(newHeight);
              } else {
                return builder_.Bind(sleep(n * n), function () {
                  return builder_.ReturnFrom(waitForActualHeight(n + 1));
                });
              }
            });
          }(singleton);
        };

        (function (arg00) {
          startImmediate(arg00);
        })(function (builder__1) {
          return builder__1.Delay(function () {
            return builder__1.Bind(waitForActualHeight(1), function (_arg2) {
              Log.trace("live", "Old height: %s, New height: %s", _this5.zoneHeight, _arg2);

              if (_this5.zoneHeight !== _arg2 ? true : patternInput[1].afterLineNumber !== dom.Line) {
                patternInput[1].afterLineNumber = dom.Line;
                patternInput[1].heightInPx = _arg2;
                _this5.zoneHeight = _arg2;

                _this5.ed.changeViewZones(function (accessor) {
                  accessor.layoutZone(patternInput[0]);
                });

                _this5.zoneSizeChanged.Trigger(null);

                return builder__1.Zero();
              } else {
                return builder__1.Zero();
              }
            });
          });
        }(singleton));
      } else {
        this.removeZone();
      }
    }
  }, {
    key: "getUpdateEventAfterChange",
    value: function () {
      var _this6 = this;

      return function (builder_) {
        return builder_.Delay(function () {
          var code = _this6.ed.getModel().getValue(1, false);

          var position = _this6.ed.getPosition();

          if (code !== _this6.lastCode) {
            _this6.lastCode = code;
            _this6.lastMapper = new LocationMapper(code);

            var loc = _this6.lastMapper.LineColToAbsolute(~~position.lineNumber, ~~position.column);

            return builder_.Bind(_this6.checker.TypeCheck(code), function (_arg3) {
              return builder_.Return(new LiveEvent("UpdateSource", [code, loc, _arg3[2], _this6.lastMapper]));
            });
          } else {
            var loc_1 = _this6.lastMapper.LineColToAbsolute(~~position.lineNumber, ~~position.column);

            return builder_.Return(new LiveEvent("UpdateLocation", [loc_1]));
          }
        });
      }(singleton);
    }
  }, {
    key: "createLivePreview",
    value: function (ed) {
      var _this7 = this;

      var liveEvent = new _Event();
      var noState = void 0;
      var $var552 = this;
      noState = _defineProperty({}, _Symbol.reflection, function () {
        return {
          interfaces: ["TheGamma.Live.Common.CustomLiveState"]
        };
      });
      var liveState = void 0;
      var Mapper = new LocationMapper("");
      var Location = 0;
      var Program = new Program_1(node_1(new _Range(0, 0), new List()));
      liveState = new LiveState(new List(), "", Program, Mapper, Location, null, noState, null);

      var applyEvent = function applyEvent(evt) {
        var liveState_1 = updateLiveState(liveState, evt);
        var newState = liveState_1.CurrentPreview == null ? null : liveState_1.CurrentPreview.Update(function (arg00) {
          liveEvent.Trigger(arg00);
        })(liveState_1)(evt);
        var newPreview = void 0;
        var $var553 = evt.Case === "UpdateSource" ? [0] : evt.Case === "UpdateLocation" ? [0] : [1];

        switch ($var553[0]) {
          case 0:
            Log.trace("live", "Searching for available previews");
            var state_1 = tryPick(function (lp) {
              var state = void 0;
              var CurrentPreview = lp;
              state = new LiveState(liveState_1.Globals, liveState_1.Code, liveState_1.Program, liveState_1.Mapper, liveState_1.Location, liveState_1.Selection, lp.InitialState, CurrentPreview);
              return lp.Update(function (arg00_1) {
                liveEvent.Trigger(arg00_1);
              })(state)(evt);
            }, _this7.livePreviews);
            newPreview = state_1;
            break;

          case 1:
            newPreview = null;
            break;
        }

        var matchValue = [newState, newPreview];
        var $var554 = matchValue[0] != null ? matchValue[0].CurrentPreview != null ? matchValue[1] != null ? matchValue[1].CurrentPreview != null ? function () {
          var st_1 = matchValue[0];
          var pr_1 = matchValue[1];
          var p2 = matchValue[1].CurrentPreview;
          var p1 = matchValue[0].CurrentPreview;
          return p1.ID !== p2.ID;
        }() ? [0, matchValue[0].CurrentPreview, matchValue[1].CurrentPreview, matchValue[1], matchValue[0]] : [1] : [1] : [1] : [1] : [1];

        switch ($var554[0]) {
          case 0:
            return $var554[3];

          case 1:
            if (matchValue[0] != null) {
              var st = matchValue[0];
              return st;
            } else if (matchValue[1] != null) {
              var pr = matchValue[1];
              return pr;
            } else {
              var CurrentPreview_1 = null;
              return new LiveState(liveState_1.Globals, liveState_1.Code, liveState_1.Program, liveState_1.Mapper, liveState_1.Location, liveState_1.Selection, noState, CurrentPreview_1);
            }

        }
      };

      add(function (evt_1) {
        try {
          Log.trace("live", "Updating state %O with event %O", liveState, evt_1);
          var oldState = liveState;
          liveState = applyEvent(evt_1);

          if ((evt_1.Case === "UpdateSource" ? false : true) ? oldState.Code !== liveState.Code : false) {
            _this7.changingEditor = true;
            ed.getModel().setValue(liveState.Code);
          }

          var matchValue_1 = liveState.Selection;

          if (matchValue_1 != null) {
            _this7.changingEditor = true;
            var mrng = {};
            mrng.startColumn = matchValue_1.StartColumn;
            mrng.startLineNumber = matchValue_1.StartLineNumber;
            mrng.endColumn = matchValue_1.EndColumn;
            mrng.endLineNumber = matchValue_1.EndLineNumber;
            ed.setSelection(mrng);
            var Selection = null;
            liveState = new LiveState(liveState.Globals, liveState.Code, liveState.Program, liveState.Mapper, liveState.Location, Selection, liveState.State, liveState.CurrentPreview);
          }

          if (_this7.changingEditor === true) {
            _this7.changingEditor = false;

            (function (arg00_2) {
              startImmediate(arg00_2);
            })(function (builder_) {
              return builder_.Delay(function () {
                Log.trace("live", "Editor changed. Getting after change event...");
                return builder_.Bind(_this7.getUpdateEventAfterChange(), function (_arg4) {
                  Log.trace("live", "Editor changed. Updating state %O with event %O", liveState, _arg4);
                  liveState = applyEvent(_arg4);
                  Log.trace("live", "Editor changed. New state %O", liveState);

                  var trigger = function trigger(arg00_3) {
                    liveEvent.Trigger(arg00_3);
                  };

                  _this7.updateZones(trigger, liveState);

                  return builder_.Zero();
                });
              });
            }(singleton));
          } else {
            var trigger_1 = function trigger_1(arg00_4) {
              liveEvent.Trigger(arg00_4);
            };

            _this7.updateZones(trigger_1, liveState);
          }
        } catch (e) {
          Log.exn("live", "Error when updating state %O with event %O: %O", liveState, evt_1, e);
        }
      }, liveEvent.Publish);

      (function (arg00_5) {
        startImmediate(arg00_5);
      })(function (builder__1) {
        return builder__1.Delay(function () {
          return builder__1.Bind(Async_AwaitFuture_Static(_this7.globals), function (_arg5) {
            liveEvent.Trigger(new LiveEvent("InitializeGlobals", [_arg5]));
            return builder__1.Zero();
          });
        });
      }(singleton));

      return function (arg00_6) {
        liveEvent.Trigger(arg00_6);
      };
    }
  }, {
    key: "ZoneSizeChanged",
    get: function () {
      return this.zoneSizeChanged.Publish;
    }
  }, {
    key: "ZoneHeight",
    get: function () {
      if (!equals(this.currentZone, null)) {
        return this.zoneHeight;
      } else {
        return 0;
      }
    }
  }]);

  return PreviewService;
}();
setType("TheGamma.Services.PreviewService", PreviewService);
//# sourceMappingURL=services.js.map